{
  Copyright 2011-2012 Joe Justice (joe@joejustice.org)
 
  This file is part of LightNet: Cloud.

  LightNet: Cloud is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  LightNet: Cloud is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with LightNet: Cloud, see files COPYING.txt and COPYING.LESSER.txt 
  If not, see <http://www.gnu.org/licenses/>.

----------------------------------------------------------------------------

Diagnostics form and tools.

}

unit diagnostics_form;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  Menus, StdCtrls, ExtCtrls, Grids, ComCtrls, lNetComponents,
  lnc_fm_unit, LCLType;

type

  { TLnC_Diagnostics }

  TLnC_Diagnostics = class(TForm)
    Button1: TButton;
    Group_Ftp: TGroupBox;
    GroupBox2: TGroupBox;
    GroupBox3: TGroupBox;
    Group_WorkingDir: TGroupBox;
    Label_wfString: TLabel;
    Label_lwConfigString: TLabel;
    Label_lwProgramString: TLabel;
    Label_ftpString: TLabel;
    Label_lwdString: TLabel;
    Label_wfReadWrite: TLabel;
    Label_lwdReadWrite: TLabel;
    Label_lwExe: TLabel;
    Label_lwConfigs: TLabel;
    Label_ftpLogin: TLabel;
    Label_ftpReadWrite: TLabel;
    Label_lwdExists: TLabel;
    Label_wfExists: TLabel;
    Timer1: TTimer;
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure Timer1Timer(Sender: TObject);
  private
    WorkDirWorking: Boolean;
    procedure CheckLWD;
    procedure CheckLFTP;
    procedure CheckLW;
    procedure CheckWF;
    { private declarations }
  public
    { public declarations }
  end; 

const
  LnC_TestFile = 'lnc_test.txt';

var
  LnC_Diagnostics: TLnC_Diagnostics;

implementation
  uses lightnet_cloud_form;

{ TLnC_Diagnostics }

procedure TLnC_Diagnostics.FormCreate(Sender: TObject);
begin
  LnC_FM.ErrorReset;
  WorkDirWorking := false;
  Label_lwdString.Caption := LnC_Settings.LocalWorkingDir;
  Label_ftpString.Caption := LnC_Settings.ftpServer + ':' + LnC_Settings.ftpPort + ' as ' +
                             LnC_Settings.ftpUser;
  Label_lwProgramString.Caption := LnC_Settings.LWScreamerFile;
  Label_lwConfigString.Caption := LnC_Settings.LWScreamerConfigDir;
  Label_wfString.Caption:= LnC_Settings.WatchFolder;

  CheckLWD;
  CheckLW;
  CheckWF;
  if WorkDirWorking then
    begin
      if LnC_FM.HasFatalError then LnC_FM.ErrorReset;
      LnC_FM.FMJobs_Add('put|' + LnC_Settings.LocalWorkingDir + PathDelim + LnC_TestFile + '|' + LnC_Settings.ftpSubDir + '/' + LnC_TestFile);
      LnC_FM.FMJobs_Add('get|' + LnC_Settings.LocalWorkingDir + PathDelim + LnC_TestFile + '|' + LnC_Settings.ftpSubDir + '/' + LnC_TestFile);
    end
  else
    begin
      Label_ftpLogin.Caption:='Can not test without Local Working Directory';
      Label_ftpReadWrite.Caption:='';
      Label_ftpLogin.Font.Color:=clRed;
    end;
end;

procedure TLnC_Diagnostics.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = LCLType.VK_ESCAPE then ModalResult := mrOK;
end;

procedure TLnC_Diagnostics.FormClose(Sender: TObject;
  var CloseAction: TCloseAction);
begin
end;

procedure TLnC_Diagnostics.CheckLWD;

var
  tf :TextFile;
  IOR: Integer;

begin
  if DirectoryExistsUTF8(LnC_Settings.LocalWorkingDir) then
    begin
      Label_lwdExists.Caption:= 'EXISTS';
      Label_lwdExists.Font.Color:=clGreen;
    end
  else
    begin
      Label_lwdExists.Caption:= 'NOT FOUND';
      Label_lwdExists.Font.Color:=clRed;
    end;

  //Test write then read
  AssignFile(tf, LnC_Settings.LocalWorkingDir + PathDelim + LnC_TestFile);
  {$I-}
  Rewrite(tf);
  {$I+}
  IOR := IOResult;
  if IOR = 0 then
    begin
      Writeln(tf,'test');
      CloseFile(tf);
      AssignFile(tf, LnC_Settings.LocalWorkingDir + PathDelim + LnC_TestFile);
      {$I-}
      Reset(tf);
      {$I+}
      IOR := IOResult;
      if IOR = 0 then
        begin
          Label_lwdReadWrite.Caption:= 'Read/Write... OK';
          Label_lwdReadWrite.Font.Color:=clGreen;
          CloseFile(tf);
          WorkDirWorking := true;
        end
      else
        begin
          Label_lwdReadWrite.Caption:= 'Read/Write... FAIL';
          Label_lwdReadWrite.Font.Color:=clRed;
          WorkDirWorking := false;
        end;
    end
  else
    begin
      Label_lwdReadWrite.Caption:= 'Read/Write... FAIL';
      Label_lwdReadWrite.Font.Color:=clRed;
      WorkDirWorking := false;
    end;
end;

procedure TLnC_Diagnostics.CheckLFTP;

begin
  if (LnC_FM.HasFatalError) or (not LnC_FM.HasJobsPending) then
    begin
      if LnC_FM.HasFatalError then
        begin
          if LnC_FM.LoginSuccess then
            begin
              Label_ftpLogin.Caption:='Login... OK';
              Label_ftpLogin.Font.Color:=clGreen;
            end
          else
            begin
              Label_ftpLogin.Caption:='Login... FAIL';
              Label_ftpLogin.Font.Color:=clRed;
            end;
          Label_ftpReadWrite.Caption:='Read/Write... FAIL';
          Label_ftpReadWrite.Font.Color:=clRed;
        end
      else
        begin
          Label_ftpLogin.Caption:='Login... OK';
          Label_ftpLogin.Font.Color:=clGreen;
          Label_ftpReadWrite.Caption:='Read/Write... OK';
          Label_ftpReadWrite.Font.Color:=clGreen;
        end;
    end
  else
    begin
      Label_ftpLogin.Caption:='Checking....';
      Label_ftpLogin.Font.Color:=clBlack;
      Label_ftpReadWrite.Caption:='Checking...';
      Label_ftpReadWrite.Font.Color:=clBlack;
    end;
end;

procedure TLnC_Diagnostics.CheckLW;

const
  lwcfgend = '.CFG';
  lwcfgmain = 'LW';
  lwcnfext = 'LWEXT';

var
  lwcfg1, lwcfg2 :string;

begin
  lwcfg1 := LnC_Settings.LWScreamerConfigDir + PathDelim + lwcfgmain +
            LnC_Settings.LWScreamerConfigSuffix + lwcfgend;
  lwcfg2 := LnC_Settings.LWScreamerConfigDir + PathDelim + lwcnfext +
            LnC_Settings.LWScreamerConfigSuffix + lwcfgend;

  if FileExistsUTF8(LnC_Settings.LWScreamerFile) then
    begin
      Label_lwExe.Caption:= 'ScreamerNet... OK';
      Label_lwExe.Font.Color:=clGreen;
    end
  else
    begin
      Label_lwExe.Caption:= 'ScreamerNet... NOT FOUND';
      Label_lwExe.Font.Color:=clRed;
    end;
  if FileExistsUTF8(lwcfg1) and FileExistsUTF8(lwcfg2) then
    begin
      Label_lwConfigs.Caption:= 'Config Files... OK';
      Label_lwConfigs.Font.Color:=clGreen;
    end
  else
    begin
      Label_lwConfigs.Caption:= 'Config Files... FAIL';
      Label_lwConfigs.Font.Color:=clRed;
    end;
end;

procedure TLnC_Diagnostics.CheckWF;

var
  tf :TextFile;
  IOR: Integer;

begin
  if DirectoryExistsUTF8(LnC_Settings.WatchFolder) then
    begin
      Label_wfExists.Caption:= 'EXISTS';
      Label_wfExists.Font.Color:=clGreen;
    end
  else
    begin
      Label_wfExists.Caption:= 'NOT FOUND';
      Label_wfExists.Font.Color:=clRed;
    end;

  //Test write then read
  AssignFile(tf, LnC_Settings.WatchFolder + PathDelim + LnC_TestFile);
  {$I-}
  Rewrite(tf);
  {$I+}
  IOR := IOResult;
  if IOR = 0 then
    begin
      Writeln(tf,'test');
      CloseFile(tf);
      AssignFile(tf, LnC_Settings.WatchFolder + PathDelim + LnC_TestFile);
      {$I-}
      Reset(tf);
      {$I+}
      IOR := IOResult;
      if IOR = 0 then
        begin
          Label_wfReadWrite.Caption:= 'Read/Write... OK';
          Label_wfReadWrite.Font.Color:=clGreen;
          CloseFile(tf);
        end
      else
        begin
          Label_wfReadWrite.Caption:= 'Read/Write... FAIL';
          Label_wfReadWrite.Font.Color:=clRed;
        end;
    end
  else
    begin
      Label_wfReadWrite.Caption:= 'Read/Write... FAIL';
      Label_wfReadWrite.Font.Color:=clRed;
    end;
end;

procedure TLnC_Diagnostics.Timer1Timer(Sender: TObject);
begin
  //Trigger FTP system
  if WorkDirWorking then
    begin
  //    LnC_FM.Run(LnC_Settings.ftpServer, LnC_Settings.ftpPort, LnC_Settings.ftpUser, LnC_Settings.ftpPassword);
      CheckLFTP;
    end;
end;

initialization
  {$I diagnostics_form.lrs}

end.

