{
  Copyright 2011-2012 Joe Justice (joe@joejustice.org)
 
  This file is part of LightNet: Cloud.

  LightNet: Cloud is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  LightNet: Cloud is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with LightNet: Cloud, see files COPYING.txt and COPYING.LESSER.txt 
  If not, see <http://www.gnu.org/licenses/>.

----------------------------------------------------------------------------

This is the foundation file for LightNet: Cloud. It creates all of the objects 
and main interface and controls the interactions.

}

unit lightnet_cloud_form;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  Menus, StdCtrls, ExtCtrls, Grids, ComCtrls, lNetComponents, LCLType,
  settings_form, diagnostics_form, settings_unit, lwscreamer_unit,
  lwscenes_unit, lnc_utils, lnc_fm_unit;

type

  { TLnC_MainForm }

  TLnC_MainForm = class(TForm)
    Button_StopLWSN: TButton;
    Button_KillLWSN: TButton;
    DiagnosticsButton: TMenuItem;
    IdleTimer_AutoLaunch: TIdleTimer;
    ftpLabel1: TLabel;
    ftpLabel2: TLabel;
    Label_Message2: TLabel;
    Label_Message1: TLabel;
    Label_Version: TLabel;
    Label_ScreamerName: TLabel;
    Button_LaunchLWSN: TButton;
    ListBox_Log: TListBox;
    loadsettingsbutton: TMenuItem;
    MainMenu1: TMainMenu;
    LoadSettingsDialog: TOpenDialog;
    ftpMemo: TMemo;
    ftpProgressBar: TProgressBar;
    SettingsMain: TMenuItem;
    SavesettingsButton: TMenuItem;
    SaveSettingsDialog: TSaveDialog;
    settingsbutton: TMenuItem;
    Timer_NewScene: TTimer;
    Timer_StatusUpdate: TTimer;
    Timer_Interface: TTimer;
    TrayIcon1: TTrayIcon;

    procedure Button_KillLWSNClick(Sender: TObject);
    procedure Button_StopLWSNClick(Sender: TObject);
    procedure DiagnosticsButtonClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
    procedure FormCreate(Sender: TObject);
    procedure Button_LaunchLWSNClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormShow(Sender: TObject);
    procedure FormWindowStateChange(Sender: TObject);
    procedure LoadSettingsbuttonClick(Sender: TObject);
    procedure SaveSettingsbuttonClick(Sender: TObject);
    procedure SettingsButtonClick(Sender: TObject);

    procedure IdleTimer_AutoLaunchTimer(Sender: TObject);
    procedure Timer_InterfaceTimer(Sender: TObject);
    procedure Timer_StatusUpdateTimer(Sender: TObject);
    procedure Timer_NewSceneTimer(Sender: TObject);
    procedure TrayIcon1DblClick(Sender: TObject);

    procedure AddToSceneLog(const Scenename, LogMessage: string);

    procedure UpdateInterface;
  private
    NumberOfFramesToRender:integer;
    FrameBatchCounter: integer;
    CloseASAP: boolean;
    procedure UpdateLog;
  public

  end;

var
  LnC_MainForm: TLnC_MainForm;
  LnC_Settings: TLnC_Settings;
  LWScreamer: TLWScreamer;
  LWScene: TLWScene;
  LnC_FM: TLnC_FM;

implementation

{ TLnC_MainForm }
procedure TLnC_MainForm.FormCreate(Sender: TObject);

var
  s: string;

  begin
    // Create the main settings class
    LnC_Settings := TLnC_Settings.Create;
    LWScreamer:= TLWScreamer.Create;
    LWScene:= TLWScene.Create;
    LnC_FM := TLnC_FM.Create;
    //Setting and/or creating the Local Working Directory
    s := ExtractFilePath(ParamStr(0)); //Get a working path from execution
    s := s + LnC_LWD_Name;
    if not DirectoryExistsUTF8(s) then CreateDir(s);
    LnC_Settings.LocalWorkingDir := s;
    //Initialize settings
    if LnC_Settings.ftpPort = '' then LnC_Settings.ftpPort := '21';
    NumberOfFramesToRender := 1;
    FrameBatchCounter := 1;
    CloseASAP := False;
    Listbox_Log.Items.Add('LightNet: Cloud launched - ' + DateTimeToStr(Now));
    Timer_NewScene.Interval := LnC_Settings.ftpCheckInterval*1000;
    IdleTimer_AutoLaunch.Interval:= LnC_Settings.AutoStartInterval*1000;
    // Fill in the lables on the main screen
    Label_Version.Caption := LnC_Settings.VersionString;
    LnC_CleanLWD(LnC_Settings.LocalWorkingDir);
    UpdateInterface;
  end;

procedure TLnC_MainForm.FormClose(Sender: TObject; var CloseAction: TCloseAction);
  begin
    LnC_Settings.Save(LnC_Settings.LnC_BaseSettingsFilename);
    if LWScreamer.IsOperational then LWScreamer.Kill;
    LnC_FM.Free;
    LnC_Settings.Free;
    LWScene.Free;
    LWScreamer.Free;
  end;

procedure TLnC_MainForm.FormCloseQuery(Sender: TObject; var CanClose: boolean);
begin
  if (LnC_FM.HasJobsPending) or (LWScreamer.IsOperational) then
    case MessageDlg('Are you sure?',
                       'LightNet: Cloud is still working. Do you want to wait for it to finish?' + LineEnding + LineEnding +
                       'Click Yes to quit when finished with current operations.' + LineEnding +
                       'Click No to quit immediately.' + LineEnding +
                       'Click Cancel to leave LightNet: Cloud open.',
                       mtConfirmation, [mbYes, mbNo, mbCancel], 0) of
      mrYes:
        begin
          CloseASAP := True;
          if LWScreamer.IsOperational then LWScreamer.Stop;
          CanClose := False;
        end;
      mrNo: CanClose := True;
      mrCancel: CanClose := False;
    end;
end;

procedure TLnC_MainForm.DiagnosticsButtonClick(Sender: TObject);
  begin
    LnC_Diagnostics := TLnC_Diagnostics.Create(self);
    LnC_Diagnostics.ShowModal;
    LnC_Diagnostics.Free;
  end;

procedure TLnC_MainForm.Button_KillLWSNClick(Sender: TObject);
  begin
    if MessageDlg('Are You sure?',
                  'This will terminate the screamer and the output will not be saved.',
                  mtConfirmation,
                  [mbYes, mbCancel],0) = mrYes
    then LWScreamer.Kill;
  end;

procedure TLnC_MainForm.Button_StopLWSNClick(Sender: TObject);
  begin
    if MessageDlg('Are You sure?',
                  'This will wait for the screamer to finish and then close it.',
                  mtConfirmation,
                  [mbYes, mbCancel],0) = mrYes
    then LWScreamer.Stop;
  end;

procedure TLnC_MainForm.UpdateInterface;

var
  FTPMessages: TStringList;
  i: integer;

  begin
    // Fill in all of the settings information
    Label_ScreamerName.Caption := LnC_Settings.ScreamerName;

    if LWScreamer.IsOperational then
      begin
        Button_LaunchLWSN.Visible:= false;
        Button_StopLWSN.Visible:=true;
        Button_KillLWSN.Visible:=true;
      end
    else
      begin
        Button_LaunchLWSN.Visible:= true;
        Button_StopLWSN.Visible:=false;
        Button_KillLWSN.Visible:=false;
        //Close the application if conditions are right
        if (CloseASAP) and (not LnC_FM.HasJobsPending) then
            if (LWScene.IsFinished) or (not LWScene.IsOperational) then Close;
        //Reset any errors if there are any
        if (CloseASAP) then LnC_FM.ErrorReset;
      end;
    if (LnC_FM.MessageLine1 = 'FTP ERROR!') or (LnC_FM.MessageLine1 = 'FTP FAILURE!') then
      begin
        Label_Message1.Font.Color:= clRed;
        Label_Message2.Font.Color:= clRed;
        Label_Message1.Caption:= 'FTP ERROR: Please review FTP messages';
        Label_Message2.Caption:= DateTimeToStr(Now);
        LnC_FM.MessageLine1 := 'Error';
        Timer_NewScene.Enabled := True;
      end;
    if (not LnC_FM.Connected) then Timer_NewScene.Enabled := True;
    if CloseASAP then
      begin
        Label_Message1.Font.Color:= clRed;
        Label_Message2.Font.Color:= clRed;
        Label_Message1.Caption:= 'LightNet: Cloud will close as soon';
        Label_Message2.Caption:= 'it is finished with current operations.';
      end;
    if (Length(LnC_FM.Messages) > 0) and (LnC_FM.Messages <> LineEnding)then
      begin
        FTPMessages := TStringList.Create;
        FTPMessages.Text := LnC_FM.Messages;
        for i := 0 to FTPMessages.Count - 1 do
          if FTPMessages.ValueFromIndex[i] <> '' then
            ftpMemo.Append(FTPMessages.ValueFromIndex[i]);
        LnC_FM.Messages := '';
        FTPMessages.Free;
      end;
    ftpProgressBar.Position := LnC_FM.Progress;
    ftpLabel1.Caption := LnC_FM.MessageLine1;
    ftpLabel2.Caption := LnC_FM.MessageLine2;
  end;

procedure TLnC_MainForm.SettingsButtonClick(Sender: TObject);
  var
    tempS: string;
    tempI, Code: integer;
    opensettings: TLnC_SettingsForm;

  begin
    opensettings := TLnC_SettingsForm.Create (self);
    //Send current Settings to the settngs form with
    with LnC_Settings do
     begin
       opensettings.Edit_ScreamerName.Text := ScreamerName;
       opensettings.Edit_LocalCache.Text := LocalCache;
       opensettings.Edit_Groups.Text := Groups;
       opensettings.Label_LocalDir.Caption := 'Local Working Directory: ' + LineEnding + LnC_Settings.LocalWorkingDir;
       opensettings.Check_Onsite.Checked := Onsite;
       opensettings.Check_Offsite.Checked := Offsite;
       opensettings.Check_AdaptiveFrames.Checked := AdaptiveFrameLoading;
       opensettings.Check_ScreamerSilent.Checked := ScreamerSilent;

       opensettings.Edit_ftpServer.Text := ftpServer;
       opensettings.Edit_ftpPort.Text := ftpPort;
       opensettings.Edit_ftpSubDir.Text := ftpSubDir;
       opensettings.Edit_ftpUser.Text := ftpUser;
       opensettings.Edit_ftpPassword.Text := ftpPassword;
       Str (ftpCheckInterval, tempS);
       opensettings.Edit_ftpCheckInterval.Text := tempS;

       Str (AutoStartInterval, tempS);
       opensettings.Edit_AutostartInterval.Text := tempS;
       opensettings.Check_AutoStart.Checked :=  Autostart;

       tempS := LWScreamerFile;
       tempI:=Length(tempS) - LWSNExeLength;
       Delete(tempS, tempI+1, LWSNExeLength);
       opensettings.Edit_LWScreamerFile.Text := tempS;
       opensettings.Edit_LWConfigDir.Text := LWScreamerConfigDir;
       opensettings.Edit_LWConfigSuffix.Text := LWScreamerConfigSuffix;

       opensettings.Edit_WatchFolder.Text := WatchFolder;
       opensettings.Check_WatchManageServer.Checked := WatchManageServer;
       opensettings.Check_WatchSilent.Checked := WatchSilent;
       Str (WatchInterval, tempS);
       opensettings.Edit_watchInterval.Text := tempS;
     end;
    if opensettings.ShowModal=mrOk then
     begin
      //If OK is clicked, update settings with settings in the Settings form
      Listbox_Log.Items.Insert(0, 'Settings Changed - ' + DateTimeToStr(Now));
      with LnC_Settings do
        begin
          ScreamerName := opensettings.Edit_ScreamerName.Text;
          LocalCache := opensettings.Edit_LocalCache.Text;
          if opensettings.Edit_Groups.Text = '' then
            Groups := 'default'
          else
            Groups := opensettings.Edit_Groups.Text;

          Onsite := opensettings.Check_Onsite.Checked;
          Offsite := opensettings.Check_Offsite.Checked;
          AdaptiveFrameLoading := opensettings.Check_AdaptiveFrames.Checked;
          ScreamerSilent := opensettings.Check_ScreamerSilent.Checked;

          ftpServer := CleanPathStr(opensettings.Edit_ftpServer.Text);
          ftpPort := opensettings.Edit_ftpPort.Text;
          ftpSubDir := CleanPathStr(opensettings.Edit_ftpSubDir.Text);
          if ftpSubDir <> '' then
            if ftpSubDir[1] <> '/' then ftpSubDir := '/' + ftpSubDir; // Add a / to the front of the FTP sub-dir in case the user did not
          ftpUser := opensettings.Edit_ftpUser.Text;
          ftpPassword := opensettings.Edit_ftpPassword.Text;
          Val (opensettings.Edit_ftpCheckInterval.Text, tempI, Code);
          if Code = 0 then ftpCheckInterval := tempI
            else ftpCheckInterval := 300;
          Timer_NewScene.Interval:=ftpCheckInterval*1000;

          Val (opensettings.Edit_AutostartInterval.Text, tempI, Code);
          if Code = 0 then AutoStartInterval := tempI
            else AutoStartInterval := 300;
          Autostart := opensettings.Check_AutoStart.Checked;

          LWScreamerFile := TrimFileName(opensettings.Edit_LWScreamerFile.Text + PathDelim + LWSNExe);
          LWScreamerConfigDir := CleanPathStr(TrimFileName(opensettings.Edit_LWConfigDir.Text));
          LWScreamerConfigSuffix := opensettings.Edit_LWConfigSuffix.Text;

          WatchManageServer := opensettings.Check_WatchManageServer.Checked;
          WatchSilent := opensettings.Check_WatchSilent.Checked;
          WatchFolder := CleanPathStr(opensettings.Edit_WatchFolder.Text);
          Val (opensettings.Edit_watchInterval.Text, tempI, Code);
          if Code = 0 then WatchInterval := tempI
            else WatchInterval := 300;

          //Save settings to base config anytime changes are made
          Save(LnC_Settings.LnC_BaseSettingsFilename);
        end;
     end;
    opensettings.Free;
    IdleTimer_AutoLaunch.Interval:= LnC_Settings.AutoStartInterval*1000;
  end;

procedure TLnC_MainForm.Timer_InterfaceTimer(Sender: TObject);
  begin
    UpdateInterface;
  end;

procedure TLnC_MainForm.Timer_StatusUpdateTimer(Sender: TObject);

  var
    i: integer;
    s, s2, s3 :string;

  begin
    //Update the local log and clear
    UpdateLog;
    //Trigger FTP system
    LnC_FM.Run(LnC_Settings.ftpServer, LnC_Settings.ftpPort, LnC_Settings.ftpUser, LnC_Settings.ftpPassword);
    //In case the scene is waiting for the scene file or the content zip and there were not
    //found on the server. Reset the scene so that it will start over on next check
    if (LnC_FM.HasFatalError) and (LWScene.IsTryingToGetSceneFiles) then LWScene.ResetScene;
    //Update Screamer and Scene
    if LWScreamer.IsOperational then LWScreamer.StatusUpdate;
    if (not LnC_FM.HasJobsPending) and (LWScene.IsOperational) then LWScene.StatusUpdate;
    //Check to see if the screamer is waiting...
    //If it is, check to see if frames have been gathered...
    //If so start rendering
    if (LWScreamer.IsWaiting) and (LWScene.HasGatheredScene) then
      begin
        FrameBatchCounter := 1;
        LWScreamer.Render (LWScene.SceneContentDir, LWScene.SceneName, LWScene.StartFrame, LWScene.EndFrame);
       end;

    //If the Screamer is on the first wait, gather 1 frame in order to get things going
    //or if it is ready, such as the content dir or scene were downloaded in last pass
    if (LWScreamer.IsFirstWait) or (LWScene.ReadyToGather) then
      if (not LnC_FM.HasJobsPending) then
        begin
          NumberOfFramesToRender := LWScene.GatherFrames(1);//Returns number of frames gathered
          if NumberOfFramesToRender < 1 then NumberOfFramesToRender := 1;
        end;

    //Update the local log and clear
    UpdateLog;

    if (LWScreamer.IsWaiting) and      //If the screamer is waiting
       (LWScene.IsFinished) and        //the scene is finished
       (not LnC_FM.HasJobsPending) then        //the FTP is not busy
      begin
        //Then figure out how many frames need to be gathered based on timer and gather them
        i := LnC_Settings.ftpCheckInterval div (FrameBatchCounter div NumberOfFramesToRender);
        i := LWScene.GatherFrames(i);//Returns number of frames gathered
        //Make sure to be sure we never divide by zero.
        //GatherFrames returns zero if the scene is not yet ready
        if i > 0 then NumberOfFramesToRender := i;
      end;

    //Check to see if the screamer has just finished. If it has, finish the scene
    if LWScreamer.IsFinishedRendering then
      begin
        LWScene.Finish;
        Str((FrameBatchCounter div 60), s2);
        Str((FrameBatchCounter mod 60), s3);
        Str(NumberOfFramesToRender, s);
        Listbox_Log.Items.Insert(0,'Rendered ' + s + ' frames in ' + s2 + ' minutes '+s3+' seconds' + ' - ' + DateTimeToStr(Now));
        AddToSceneLog(LWScene.BaseSceneName, 'Renderd ' + s + ' frames in ' + s2 + ' minutes '+s3+' seconds' + ' - ' + DateTimeToStr(Now));
        LnC_FM.ErrorReset; // In case there was a timeout or some other problem
        LWScene.Uploadlog(LnC_LogName(LWScene.BaseSceneName, LnC_Settings.ScreamerName));
      end;

    //Add to timer if rendering
    if LWScreamer.IsRendering then Inc(FrameBatchCounter);

  end;

procedure TLnC_MainForm.Timer_NewSceneTimer(Sender: TObject);
begin
  Timer_NewScene.Enabled := False;
  if not CloseASAP then
    begin
      //If there was a fatal error, reset so LnC can try again
      LnC_FM.ErrorReset;
      //If the Scene was still working on something when an error occured
      if (LWScene.IsOperational) then LWScene.StatusUpdate;
      //If the scene is waiting and the screamer is operational
      if (LWScreamer.IsWaiting) and (not LWScene.IsOperational) then
          NumberOfFramesToRender := LWScene.GatherFrames(1);//Returns number of frames gathered
      if NumberOfFramesToRender < 1 then NumberOfFramesToRender := 1;
      //Trigger FTP system
//      LnC_FM.Run(LnC_Settings.ftpServer, LnC_Settings.ftpPort, LnC_Settings.ftpUser, LnC_Settings.ftpPassword);
    end;
end;

procedure TLnC_MainForm.Button_LaunchLWSNClick(Sender: TObject);
begin
  //If there was a fatal error, reset so LnC can try again
  if LnC_FM.HasFatalError then LnC_FM.ErrorReset;

  if not DirectoryExistsUTF8(LnC_Settings.LocalWorkingDir) then
    begin
      Label_Message1.Font.Color:= clRed;
      Label_Message2.Font.Color:= clRed;
      Label_Message1.Caption:= 'ERROR: Local working directory';
      Label_Message2.Caption:= 'Not properly set';
    end
  else
    LWScreamer.Launch (LnC_Settings.LWScreamerFile,
                       LnC_Settings.LWScreamerConfigDir,
                       LnC_Settings.LocalWorkingDir,
                       LnC_Settings.ScreamerSilent );
end;

procedure TLnC_MainForm.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  //Launch Screamer hotkey
  if (Key = LCLType.VK_S) and (ssCtrl in Shift) and (not LWScreamer.IsOperational)
    then Button_LaunchLWSNClick(Sender);
  //Quit hotkey
  if (Key = LCLType.VK_Q) and (ssCtrl in Shift) then
    begin
      CloseASAP := True;
      if LWScreamer.IsOperational then LWScreamer.Stop;
    end;
  //Force check hotkey
  if (Key = LCLType.VK_F) and (ssCtrl in Shift) then Timer_NewSceneTimer(Sender);
end;

procedure TLnC_MainForm.FormShow(Sender: TObject);
var
  s: string;
begin
  if not DirectoryExistsUTF8(LnC_Settings.WatchFolder) then
    begin
      MessageDlg('There may be a problem.', 'Watch Folder not found. Please check the setting.', mtInformation, [mbOK], 0);
    end;
  s := '';
  if (LnC_Settings.ftpServer = '') or (LnC_Settings.ftpPort = '') or
     (LnC_Settings.ftpUser = '') or (LnC_Settings.ftpPassword = '')
  then
    begin
      s := 'FTP is not properly set.' + LineEnding;
    end;
  if (not FileExistsUTF8(LnC_Settings.LWScreamerFile)) or (not DirectoryExistsUTF8(LnC_Settings.LWScreamerConfigDir)) then
    begin
      s := s + 'LightWave not properly setup.' + LineEnding;
    end;
  if s <> '' then
    begin
      MessageDlg('We have a problem.', s, mtError, [mbOK], 0);
      settingsbuttonClick(self);
    end;
end;

procedure TLnC_MainForm.IdleTimer_AutoLaunchTimer(Sender: TObject);
  begin
    if not DirectoryExistsUTF8(LnC_Settings.LocalWorkingDir) then
      begin
        Label_Message1.Font.Color:= clRed;
        Label_Message2.Font.Color:= clRed;
        Label_Message1.Caption:= 'ERROR: Local working directory';
        Label_Message2.Caption:= 'Not properly set';
      end
    else
      if (not LWScreamer.IsOperational) and (LnC_Settings.Autostart) and (not CloseASAP) then
        begin
          LWScreamer.Launch (LnC_Settings.LWScreamerFile,
                             LnC_Settings.LWScreamerConfigDir,
                             LnC_Settings.LocalWorkingDir,
                             LnC_Settings.ScreamerSilent);
        end;
  end;

procedure TLnC_MainForm.LoadSettingsbuttonClick(Sender: TObject);
  begin
    if LoadSettingsDialog.Execute then LnC_Settings.Load(LoadSettingsDialog.Filename);

    //Save settings to base config right after they are changed
    LnC_Settings.Save(LnC_Settings.LnC_BaseSettingsFilename);
  end;

procedure TLnC_MainForm.SaveSettingsbuttonClick(Sender: TObject);
  var
    f, s: string; //Filename to save to

  begin
    s := '';
    f := '';
    if SaveSettingsDialog.Execute then
      begin
        f := SaveSettingsDialog.Filename;
        //Check to see if .conf is on the filname and add if it is not
        s := Copy (f, (Length(f) - 4), 5);
        if s <> LnC_SettingsFilenameExtention then
          f := f + LnC_SettingsFilenameExtention;
        LnC_Settings.Save(f)
      end;
  end;

procedure TLnC_MainForm.AddToSceneLog(const SceneName, LogMessage:string);
  var
    FullLogFile:string;
    LogFile: TextFile;
    IOR: Integer;
    s:string;

  begin
    s := LnC_LogName(Scenename, LnC_Settings.ScreamerName);
    FullLogFile := LnC_Settings.LocalWorkingDir+PathDelim + s;

    try
      AssignFile(logfile, FullLogFile);
      {$I-}
      Reset(LogFile);
      {$I+}
      IOR := IOResult;
      if IOR = 2 then
        begin
          Rewrite(LogFile);
          Writeln(LogFile, LogMessage);
        end
      else
        begin
          Append(logfile);
          Writeln(LogFile, LogMessage);
        end;
    finally
      CloseFile(LogFile);
    end;

  end;

procedure TLnC_MainForm.TrayIcon1DblClick(Sender: TObject);
begin
  TrayIcon1.Hide;
  LnC_MainForm.WindowState := wsNormal;
  LnC_MainForm.Show;
end;

procedure TLnC_MainForm.FormWindowStateChange(Sender: TObject);
begin
  If LnC_MainForm.WindowState = wsMinimized then
    begin
      TrayIcon1.Show;
      LnC_MainForm.Hide;
    end;
end;

procedure TLnC_MainForm.UpdateLog;

begin
    //Message for user that don't need to be in log
    if LWScene.LogMsg = 'no frames' then
      begin
        Label_Message1.Font.Color:= clBlack;
        Label_Message2.Font.Color:= clBlack;
        Label_Message1.Caption:= 'No frames as of last check:';
        Label_Message2.Caption:= DateTimeToStr(Now);
        LWScene.LogMsg := '';
        Timer_NewScene.Enabled := True;
      end;
    if LWScreamer.LogMsg = 'ERROR: settings' then
      begin
        Label_Message1.Font.Color:= clRed;
        Label_Message2.Font.Color:= clRed;
        Label_Message1.Caption:= 'Screamernet ERROR';
        Label_Message2.Caption:= 'Settings no properly set.';
        LWScreamer.LogMsg := '';
      end;
     if LWScreamer.LogMsg = 'Terminated' then
      begin
        Label_Message1.Font.Color:= clBlack;
        Label_Message2.Font.Color:= clBlack;
        Label_Message1.Caption:= 'ScreamerNet Terminated';
        Label_Message2.Caption:= DateTimeToStr(Now);
        LWScreamer.LogMsg := '';
      end;
     if LWScreamer.LogMsg = 'Offline' then
      begin
        Label_Message1.Font.Color:= clBlack;
        Label_Message2.Font.Color:= clBlack;
        Label_Message1.Caption:= 'ScreamerNet Offline';
        Label_Message2.Caption:= DateTimeToStr(Now);
        LWScreamer.LogMsg := '';
      end;
     if LWScreamer.LogMsg = 'ERROR: crashed' then
      begin
        Label_Message1.Font.Color:= clRed;
        Label_Message2.Font.Color:= clRed;
        Label_Message1.Caption:= 'ERROR: ScreamerNet manually closed or crashed';
        Label_Message2.Caption:= DateTimeToStr(Now);
        LWScreamer.LogMsg := '';
      end;

    //Catch-all error. Start timer.
    if LWScene.LogMsg <> '' then
      if Copy(LWScene.LogMsg, 1, 5) = 'ERROR' then
        begin
          Label_Message1.Font.Color:= clRed;
          Label_Message2.Font.Color:= clRed;
          Label_Message1.Caption:= 'There is a problem with the scene';
          Label_Message2.Caption:= DateTimeToStr(Now);
          Timer_NewScene.Enabled := True;
        end;

    //Update the local log and clear
    if LWScreamer.LogMsg <> '' then
      begin
        Label_Message1.Font.Color:= clBlack;
        Label_Message2.Font.Color:= clBlack;
        Label_Message1.Caption:= 'Operational';
        Label_Message2.Caption:= '';
        Listbox_Log.Items.Insert(0, LWScreamer.LogMsg + ' - ' + DateTimeToStr(Now));
        if LWScene.SceneName = '' then
          AddToSceneLog('', LWScreamer.LogMsg + ' - ' + DateTimeToStr(Now))
        else
          AddToSceneLog(LWScene.BaseSceneName, LWScreamer.LogMsg + ' - ' + DateTimeToStr(Now));
        LWScreamer.LogMsg := '';
      end;

    //Update the local log and clear
    if LWScene.LogMsg <> '' then
      begin
        Label_Message1.Font.Color:= clBlack;
        Label_Message2.Font.Color:= clBlack;
        Label_Message1.Caption:= 'Operational';
        Label_Message2.Caption:= '';
        Listbox_Log.Items.Insert(0, LWScene.LogMsg + ' - ' + DateTimeToStr(Now));
        if LWScene.SceneName = '' then
          AddToSceneLog('', LWScene.LogMsg + ' - ' + DateTimeToStr(Now))
        else
          AddToSceneLog(LWScene.BaseSceneName, LWScene.LogMsg + ' - ' + DateTimeToStr(Now));
        LWScene.LogMsg := '';
      end;
  if Listbox_Log.Count > 8 then
    begin

    end;
end;

initialization
  {$I lightnet_cloud_form.lrs}

end.

