{
  Copyright 2011-2012 Joe Justice (joe@joejustice.org)
 
  This file is part of LightNet: Cloud.

  LightNet: Cloud is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  LightNet: Cloud is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with LightNet: Cloud, see files COPYING.txt and COPYING.LESSER.txt 
  If not, see <http://www.gnu.org/licenses/>.

----------------------------------------------------------------------------

Some functions and variables used by both Cloud and Watcher.

}

unit lnc_utils;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Dialogs;

const
  LnC_VersionMain = 1;
  LnC_VersionSub = 8;
  LnC_VersionSubSub = 0;
  LnC_VersionName = ' LightNet: Cloud | Copyright 2011-2012 Joe Justice | www.joejustice.org | Win 7';
  LnC_Date = '2 | 29 | 2012';
  LnC_SettingsFilenameExtention = '.conf';
  LnC_NoSetting = '(none)';
  ftpPathDelim = '/';
  LnC_RGB_Suffix      = '_rgb_';
  LnC_Apha_Suffix     = '_alpha_';
  LnC_OutputDir_Suffix= '_output';
  LnC_LWD_Name = 'lnc_localworkingdir';
  LnC_LastUpdateFile = 'lastupdate.txt';

  LWSNExe = 'lwsn.exe';
  LWSNExeLength = 9;

  OnsiteSceneINI = '0_lnc_scenes_onsite.ini';
  OffsiteSceneINI = '0_lnc_scenes_offsite.ini';

function LnC_GetftpOutputDir (const SceneName, Sub:string):string;
function LnC_GetOutputDir (const SceneName, Sub:string):string;
function LnC_LogName(const SceneName, ScreamerName:string):string;
procedure LnC_CleanLWD(const LWD:string);
function LnC_ftpExtractFileName(const s: string): string;
function LnC_MakeLastUpdate(const Location, Name:string): string;

implementation

function LnC_ftpExtractFileName(const s: string): string;
var
  i:integer;
begin
  i := Length(s);
  repeat
    i := i - 1;
  until (i < 0) or (s[i] = '/');
  if i > 0 then Result := Copy(s, i + 1, Length(s))
  else Result := s;

end;

function LnC_GetftpOutputDir (const SceneName, Sub:string):string;
var
  s, sn:string;
  strim:integer;

begin
  sn := SceneName;
  s:='';

  //Remove .lws or _seg info
  strim := Length(sn);
  if Copy(sn, strim - 7, 8) = '_seg.lws' then strim := strim - 11
    else strim := strim - 4;
  sn := Copy(sn, 1, strim);

  if Sub = 'root' then s := sn + LnC_OutputDir_Suffix;
  if Sub = 'rgb' then s:= sn + LnC_OutputDir_Suffix + ftpPathDelim + 'rgb';
  if Sub = 'alpha' then s:= sn + LnC_OutputDir_Suffix + ftpPathDelim + 'alpha';
  Result := s;
end;

function LnC_GetOutputDir (const SceneName, Sub:string):string;
var
  s, sn:string;
  strim:integer;

begin
  s:='';
  sn := SceneName;

  //Remove .lws or _seg info
  strim := Length(sn);
  if Copy(sn, strim - 7, 8) = '_seg.lws' then strim := strim - 11
    else strim := strim - 4;
  sn := Copy(sn, 1, strim);

  if sub = 'root' then s := sn + LnC_OutputDir_Suffix;
  if sub = 'rgb' then s:= sn + LnC_OutputDir_Suffix + PathDelim + 'rgb';
  if sub = 'alpha' then s:= sn + LnC_OutputDir_Suffix + PathDelim + 'alpha';
  Result := s;
end;

function LnC_LogName(const SceneName, ScreamerName:string):string;
  var
    i :integer;
    s : string;

  begin
    //A little work to sure the log filenames are legal and consistant
    s := scenename;
    s := s + '_' + ScreamerName;

    //Change \ to _
    for i:=1 to Length(s) do
      if s[i] = '\' then s[i] := '_';
    //Change / to _
    for i:=1 to Length(s) do
      if s[i] = '/' then s[i] := '_';
    //Change : to _
    for i:=1 to Length(s) do
      if s[i] = ':' then s[i] := '_';
    //Change space to _
    for i:=1 to Length(s) do
      if s[i] = ' ' then s[i] := '_';
    //Change . to _
    for i:=1 to Length(s) do
      if s[i] = '.' then s[i] := '_';
    Result := Lowercase(s) + '.log';
  end;

procedure LnC_CleanLWD(const LWD:string);

var
  OutputFiles: TSearchRec;
  DoubleCheck: Boolean;


begin
  //Delete everything in LWD
  //First make sure it exists
  DoubleCheck := true;

  if DirectoryExists(LWD) then
    begin
      //Delete the directories
      if FindFirst (LWD + PathDelim + '*', faDirectory, OutputFiles) = 0 then
        begin
          Repeat
            With OutputFiles do
              begin
                if ((Attr and faDirectory) = faDirectory) and (Name <> '.') and (Name <> '..') then
                    DeleteDirectory(LWD + PathDelim + Name, false);
              end;
          Until FindNext(OutputFiles)<>0;
        end;
      FindClose(OutputFiles);

      //Then delete any files
      If FindFirst (LWD + PathDelim + '*', faAnyFile - faDirectory, OutputFiles) = 0 then
        begin
        Repeat
          With OutputFiles do
            begin
              if (Name <> 'job') and (Name <> 'ack') then DeleteFileUTF8(LWD + PathDelim + Name);
            end;
        Until FindNext(OutputFiles)<>0;
        end;
      FindClose(OutputFiles);

    end;
end;

function LnC_MakeLastUpdate(const Location, Name:string): string;
var
  IOR:integer;
  s: string;
  LastUpdateFile: TextFile;

begin
  s := Location + LnC_LastUpdateFile;
  if FileExistsUTF8(s) then DeleteFileUTF8(s);
  AssignFile(LastUpdateFile, s);
  {$I-}
    Rewrite(LastUpdateFile);
  {$I+}
  IOR := IOResult;
  if IOR = 0 then
    begin
      //Put the time and the screamer name in there
      Writeln(LastUpdateFile, TimeToStr(Now) + ' - Last updated by ' + Name);
      CloseFile(LastUpdateFile);
    end;

  Result := s;
end;

end.

