{
  Copyright 2011-2012 Joe Justice (joe@joejustice.org)
 
  This file is part of LightNet: Cloud.

  LightNet: Cloud is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  LightNet: Cloud is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with LightNet: Cloud, see files COPYING.txt and COPYING.LESSER.txt 
  If not, see <http://www.gnu.org/licenses/>.

----------------------------------------------------------------------------

LightNet: Watcher main form and system.

}

unit lncwatcher_form;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, EditBtn, Grids, lNetComponents, IniFiles,
  LCLType, Menus, settings_unit, lnc_utils, lnc_fm_unit, watcher_add_form,
  watcher_top_form, watcher_remove_form, watcher_gapcheck_form,
  watcher_filelister_form, watcher_scenepurge_form, watcher_scenedownload_form;

type

  { TLnCWatch }

  TLnCWatch = class(TForm)
    ForceCheck: TMenuItem;
    ftpLabel1: TLabel;
    ftpLabel2: TLabel;
    MainMenu1: TMainMenu;
    Memo_FTPmsg: TMemo;
    ManageServer: TMenuItem;
    InitializeServer: TMenuItem;
    OutputDownload: TMenuItem;
    ScenePurge: TMenuItem;
    Server: TMenuItem;
    SceneGapCheck: TMenuItem;
    SceneAdd: TMenuItem;
    MenuItem2: TMenuItem;
    SceneRemove: TMenuItem;
    SceneTop: TMenuItem;
    Queue: TMenuItem;
    Label_Version: TLabel;
    Label_LastCheck: TLabel;
    ftpProgressBar: TProgressBar;
    Grid_Scene: TStringGrid;
    INI_Timer: TTimer;
    ETA_Timer: TTimer;
    FTP_Timer: TTimer;
    Panel1: TPanel;
    TrayIcon1: TTrayIcon;
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormShow(Sender: TObject);
    procedure FormWindowStateChange(Sender: TObject);
    procedure ETA_TimerTimer(Sender: TObject);
    procedure FTP_TimerTimer(Sender: TObject);
    procedure InitializeServerClick(Sender: TObject);
    procedure INI_TimerTimer(Sender: TObject);
    procedure ManageServerClick(Sender: TObject);
    procedure OutputDownloadClick(Sender: TObject);
    procedure SceneAddClick(Sender: TObject);
    procedure SceneGapCheckClick(Sender: TObject);
    procedure ScenePurgeClick(Sender: TObject);
    procedure SceneRemoveClick(Sender: TObject);
    procedure SceneTopClick(Sender: TObject);
    procedure TrayIcon1DblClick(Sender: TObject);
    procedure UpdateEverything;
    procedure FormCreate(Sender: TObject);
  private
    TimeCounter:LongWord;
    LastQueueOn, LastQueueOff: TStringList;
    IsUpdating, CloseASAP: boolean;

    { private declarations }
    procedure SegmentScenes(const SceneDir:string);
    procedure SegmentMaker(const SceneFile:TStringList; const i: integer; const RegionLine, ScenePathBase, Suffix: string);
    procedure CreateNewINIFiles(const OldININame: string; const NewININame: string; const CmdTXT: string);
    function MoveWatchedFiles(const WildCardExt:string): boolean;
    procedure UploadQueue(const QueueLocation: string);
  public
    RootDir: string;
    { public declarations }
  end; 

const
  OnsiteCmdTXT = '_on_site.txt';
  OffsiteCmdTXT = '_off_site.txt';
  WatcherUploadQueue = 'upload';
  oldOnsiteSceneINI = 'old_lnc_scenes_onsite.ini';
  oldOffsiteSceneINI = 'old_lnc_scenes_offsite.ini';

var
  LnCWatch: TLnCWatch;
  LnC_Settings: TLnC_Settings;
  LnC_FM: TLnC_FM;


implementation

{ TLnCWatch }

procedure TLnCWatch.FormCreate(Sender: TObject);

begin
  // Create the main settings classes
  LnC_Settings := TLnC_Settings.Create;
  if LnC_Settings.ftpPort = '' then LnC_Settings.ftpPort := '21';
  LnC_FM := TLnC_FM.Create;
  // Initiialize settings
  INI_Timer.Interval := LnC_Settings.WatchInterval*1000;
  Label_Version.Caption := LnC_Settings.VersionString;
  RootDir := ExtractFilePath(ParamStr(0)); //Get a working path from execution
  RootDir := Copy(RootDir, 1, (Length(RootDir) - 1)); //Clip off trailing path delim
  TrayIcon1.Hide;
  CloseASAP := False;
  ManageServer.Checked := LnC_Settings.WatchManageServer;
  // Queue and time estimation variabes
  TimeCounter := 0;
  LastQueueOn := TStringList.Create;
  LastQueueOn.Add('0|0|0|0|0|0|0');
  LastQueueOff := TStringList.Create;
  LastQueueOff.Add('0|0|0|0|0|0|0');

  IsUpdating := False;
  if not DirectoryExistsUTF8(LnC_Settings.WatchFolder) then
    begin
      INI_Timer.Enabled := false;
      FTP_Timer.Enabled := false;
    end
  else
    INI_TimerTimer(nil);

end;
procedure TLnCWatch.FormShow(Sender: TObject);
begin
  if not DirectoryExistsUTF8(LnC_Settings.WatchFolder) then
    begin
      MessageDlg('We have a problem.', 'Watch Folder not found. Please check the setting in LightNet: Cloud',mtError, [mbOK], 0);
      Close;
    end;
end;
procedure TLnCWatch.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  LnC_Settings.Free;
  LnC_FM.Free;
end;

procedure TLnCWatch.FormCloseQuery(Sender: TObject; var CanClose: boolean);
begin
  if (LnC_FM.HasJobsPending) then
    case MessageDlg('Are you sure?',
                       'LightNet: Watcher is still working. Do you want to wait for it to finish?' + LineEnding + LineEnding +
                       'Click Yes to quit when finished with current operations.' + LineEnding +
                       'Click No to quit immediately.' + LineEnding +
                       'Click Cancel to leave LightNet: Watcher open.',
                       mtConfirmation, [mbYes, mbNo, mbCancel], 0) of
      mrYes:
        begin
          CloseASAP := True;
          CanClose := False;
        end;
      mrNo: CanClose := True;
      mrCancel: CanClose := False;
    end;
end;

procedure TLnCWatch.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if (Key = LCLType.VK_Q) and (ssCtrl in Shift) then CloseASAP := True;
  if (Key = LCLType.VK_F) and (ssCtrl in Shift) then INI_TimerTimer(Sender);
end;

procedure TLnCWatch.FormWindowStateChange(Sender: TObject);
begin
  If LnCWatch.WindowState = wsMinimized then
    begin
      TrayIcon1.Show;
      LnCWatch.Hide;
    end;
end;

procedure TLnCWatch.TrayIcon1DblClick(Sender: TObject);
begin
  TrayIcon1.Hide;
  LnCWatch.WindowState := wsNormal;
  LnCWatch.Show;
end;

procedure TLnCWatch.ETA_TimerTimer(Sender: TObject);
begin
   // Inc the FrameCounter for telling how many frames have rendered since last check
   Inc(TimeCounter);
end;

procedure TLnCWatch.CreateNewINIFiles(const OldININame: string; const NewININame: string; const CmdTXT: string);

var
  OriginalINI, NewINI:TIniFile;

  CmdsTXTFile, F: TextFile;
  CmdLine : string;
  IOR: Integer;

  NewINIAsTXT :TStringList;

  SceneContentDir, SceneName, Groups:string;

  i, INICntr, NewINICntr, StartFrame, EndFrame, AverageFrame :integer;
  INICntrS, NewINICntrS:string;

  s:string; //Temp string for testing things

  top1, top2, top3, top4, top5, top6, top7 :string;

begin
  i := 0;
  IOR := 0;
  INICntrS:= '';
  INICntr:=0;
  NewINICntr:=0;
  NewINICntrS:='';
  StartFrame := 1;
  EndFrame := 0;
  s := '';
  top1 := '';
  top2 := '';
  top3 := '';
  top4 := '';
  top5 := '';
  top6 := '';
  top7 := '';

  OriginalINI := TIniFile.Create(OldININame);
  NewINI := TIniFile.Create(NewININame);

  //Delete the old newINI if it already exists and create a new one
  if FileExistsUTF8(NewININame) then DeleteFileUTF8(NewININame);
  AssignFile(f, NewININame);
  Rewrite(f);
  Writeln(f,'');
  CloseFile(f);

  //Transfer the current ini into a new ini while removing completed scenes
  while s <> '[none]'  do
  begin
    Inc(INICntr);
    Str(INICntr, INICntrS);
    //Check to see if there is anything in a section first
    s  := OriginalINI.ReadString(INICntrS, 'StartFrame', '[none]');
    SceneName  := OriginalINI.ReadString(INICntrS, 'SceneName', '[none]');
    SceneContentDir  := OriginalINI.ReadString(INICntrS, 'SceneContentDir', '[none]');
    StartFrame  := OriginalINI.ReadInteger(INICntrS, 'StartFrame', 1);
    EndFrame := OriginalINI.ReadInteger(INICntrS, 'EndFrame', 0);
    AverageFrame := OriginalINI.ReadInteger(INICntrS, 'AverageFrame', 0);
    Groups  := OriginalINI.ReadString(INICntrS, 'Groups', 'default');

    //Is so, then poulate the information in the new ini file
    if (s <> '[none]') and (StartFrame <= EndFrame) then
      begin
        Inc(NewINICntr);
        Str(NewINICntr, NewINICntrS);
        NewINI.WriteString(NewINICntrS, 'SceneName', SceneName);
        NewINI.WriteString(NewINICntrS, 'SceneContentDir', SceneContentDir);
        NewINI.WriteInteger(NewINICntrS, 'StartFrame', StartFrame);
        NewINI.WriteInteger(NewINICntrS, 'EndFrame', EndFrame);
        NewINI.WriteInteger(NewINICntrS, 'AverageFrame', AverageFrame);
        NewINI.WriteString(NewINICntrS, 'Groups', Groups);
      end;
  end;
  OriginalINI.Free; //Done with the original
  if FileExistsUTF8(OldININame) then DeleteFileUTF8(OldININame);
  try
    AssignFile(CmdsTXTFile, CmdTXT);
    {$I-}
    Reset(CmdsTXTFile);
    {$I+}
    IOR := IOResult;
    if IOR = 0 then
      begin
//Prcoess add commands
        s := '';
        Repeat
          Readln(CmdsTXTFile,CmdLine);
          CmdLine := Lowercase(CmdLine);
          if CmdLine = 'add' then
            begin
              Inc(NewINICntr);
              Str(NewINICntr, NewINICntrS);
              Readln(CmdsTXTFile,CmdLine);
              Readln(CmdsTXTFile,s);
              NewINI.WriteString(NewINICntrS, 'SceneName', Lowercase(CmdLine));
              NewINI.WriteString(NewINICntrS, 'SceneContentDir', LowerCase(s));
              Readln(CmdsTXTFile,CmdLine);
              NewINI.WriteString(NewINICntrS, 'StartFrame', CmdLine);
              Readln(CmdsTXTFile,CmdLine);
              NewINI.WriteString(NewINICntrS, 'EndFrame', CmdLine);
              Readln(CmdsTXTFile,CmdLine);
              NewINI.WriteInteger(NewINICntrS, 'AverageFrame', 0);
              NewINI.WriteString(NewINICntrS, 'Groups', CmdLine);
            end;
        Until Eof(CmdsTXTFile);
//-----End add command
//Process remove commands
        Reset(CmdsTXTFile);
        //Rest variables used earlier
        INICntr := 0;
        s := '';
        SceneName := '';

        //Get a count of how many sections there are
        while S <> '[none]'  do
          begin
            Inc(INICntr);
            Str(INICntr, INICntrS);
            S := NewINI.ReadString(INICntrS, 'SceneName', '[none]');
        end;
        //Now check the file
        Repeat
          Readln(CmdsTXTFile, CmdLine);
          CmdLine := Lowercase(CmdLine);
          if CmdLine = 'remove' then
            begin
              //Get Queue name
              Readln(CmdsTXTFile,CmdLine);
              CmdLine := Lowercase(CmdLine);
              //Compare it to all Queue names in the ini file
              //If found, delete the section
              ///-------------Scane and remove
              for i := 1 to INICntr do
                begin
                  Str(i, INICntrS);
                  SceneName := NewINI.ReadString(INICntrS, 'SceneName', '[none]');
                  SceneName := Lowercase(SceneName);
                  if SceneName = CmdLine then NewINI.EraseSection(INICntrS);
                end;
            end;
              ///-------------Scane and remove
        Until Eof(CmdsTXTFile);
//-----End Remove command
        CloseFile(CmdsTXTFile);
      end
    else
      begin
        //Create the file for future use
        try
          AssignFile(CmdsTXTFile, CmdTXT);
          Rewrite(CmdsTXTFile);
          Writeln(CmdsTXTFile,'');
        finally
          CloseFile(CmdsTXTFile);
        end;
      end;
    finally
    end;
  //Finished with the ini tools need to reformat with standard text
  NewINI.Free;
//Process the "top" command
  AssignFile(CmdsTXTFile, CmdTXT);
  Reset(CmdsTXTFile);
  NewINIAsTXT := TStringList.Create;
  NewINIAsTXT.LoadFromFile(NewININame);
  s := '';
  Repeat
    Readln(CmdsTXTFile, CmdLine);
    CmdLine := Lowercase(CmdLine);
    if CmdLine = 'top' then
      begin
        //Get Queue name
        Readln(CmdsTXTFile,CmdLine);
        CmdLine := Lowercase(CmdLine);
        //Compare it to all Queue names in the ini file
        //If found, move the string above and three below to the top
        for i := 0 to NewINIAsTXT.Count-1 do
          begin
            s := NewINIAsTXT[i];
            if s = ('SceneName=' + CmdLine) then
              begin
                //Move string to top
                top1 := NewINIAsTXT[(i-1)];
                top2 := NewINIAsTXT[(i)];
                top3 := NewINIAsTXT[(i+1)];
                top4 := NewINIAsTXT[(i+2)];
                top5 := NewINIAsTXT[(i+3)];
                top6 := NewINIAsTXT[(i+4)];
                top7 := NewINIAsTXT[(i+5)];

                NewINIAsTXT.Delete(i-1);
                NewINIAsTXT.Delete(i-1);
                NewINIAsTXT.Delete(i-1);
                NewINIAsTXT.Delete(i-1);
                NewINIAsTXT.Delete(i-1);
                NewINIAsTXT.Delete(i-1);
                NewINIAsTXT.Delete(i-1);

                NewINIAsTXT.Insert(0, top1);
                NewINIAsTXT.Insert(1, top2);
                NewINIAsTXT.Insert(2, top3);
                NewINIAsTXT.Insert(3, top4);
                NewINIAsTXT.Insert(4, top5);
                NewINIAsTXT.Insert(5, top6);
                NewINIAsTXT.Insert(6, top7);
                NewINIAsTXT.Insert(7, '');
              end;
          end;
      end;
        ///-------------Scane and remove
  Until Eof(CmdsTXTFile);
  CloseFile(CmdsTXTFile);
  NewINIAsTXT.SaveToFile(NewININame);
  NewINIAsTXT.Free;
//-----End Top command

  //Make sure sections have proper numbering
  INICntr := 0;
  INICntrS := '';
  s := '';
  try
    NewINIAsTXT := TStringList.Create;
    NewINIAsTXT.LoadFromFile(NewININame);
    for i := 0 to NewINIAsTXT.Count-1 do
      begin
        s := NewINIAsTXT[i];
        if s <> '' then
          if s[1] = '[' then
            begin
              Inc(INICntr);
              Str(INICntr, INICntrS);
              NewINIAsTXT.Delete(i);
              NewINIAsTXT.Insert(i, '[' + INICntrS + ']');
            end
      end;
    // Add last LnC info so the file will never be totally empty unless there is a problem
    NewINIAsTXT.Insert(0, '');
    NewINIAsTXT.Insert(0, 'Version=' + LnC_Settings.VersionString);
    NewINIAsTXT.Insert(0, '[LnC]');
  finally
    NewINIAsTXT.SaveToFile(NewININame);
    NewINIAsTXT.Free;
  end;
  //Clear command file
  try
    AssignFile(CmdsTXTFile, CmdTXT);
    Rewrite(CmdsTXTFile);
    Writeln(CmdsTXTFile,'');
  finally
    CloseFile(CmdsTXTFile);
  end;
end;

function TLnCWatch.MoveWatchedFiles(const WildCardExt:string): boolean;

var
  WatchedFiles: TSearchRec;
  s: string;
  i: integer;

begin
  Result := false; //Assume no files were copied
  //Get a listing of all wildcare files
  If FindFirst (LnC_Settings.WatchFolder + PathDelim + WildCardExt, faAnyFile - faDirectory, WatchedFiles)=0 then
    begin
    Result := true; //Establish files were moved
    Repeat
      With WatchedFiles do
        begin
          //Create upload queue dir if it doesn't exist
          if not DirectoryExistsUTF8(LnC_Settings.WatchFolder + PathDelim + WatcherUploadQueue) then CreateDir(LnC_Settings.WatchFolder + PathDelim + WatcherUploadQueue);
          //Remove Spaces in name
          s := Name;
          s := Lowercase(s);
          //Change space to _
          for i:=1 to Length(s) do if s[i] = ' ' then s[i] := '_';
          //move files to the upload queue
          RenameFileUTF8(LnC_Settings.WatchFolder + PathDelim + Name, LnC_Settings.WatchFolder + PathDelim + WatcherUploadQueue + PathDelim + s);
        end;
    Until FindNext(WatchedFiles)<>0;
    end;
  FindClose(WatchedFiles);
end;

procedure TLnCWatch.UploadQueue(const QueueLocation: string);
var
  QueueFiles: TSearchRec;
  ftpOutputdir, ftpOutputdirRGB,ftpOutputdirAlpha:string;
  SceneFile: TStringList;
  OnsiteCmdsTXTFile:TextFile;
  s, ScnName, ScnContent, ScnFirstFrame, ScnLastFrame, Groups : string;
  i, IOR, b :integer;

begin
  LnC_FM.ErrorReset;
  ScnName:= '';
  ScnContent:= '';
  ScnFirstFrame:= '';
  ScnLastFrame:= '';
  Groups := '';
  s:='';
  i := 0;

  //First upload zip and delete files
  If FindFirst (QueueLocation + PathDelim + '*.zip', faAnyFile - faDirectory, QueueFiles)=0 then
    begin
      Repeat
        With QueueFiles do
          begin
            RenameFileUTF8(QueueLocation + PathDelim + Name, QueueLocation + PathDelim + Lowercase(Name));
            LnC_FM.FMJobs_Add('put|' + QueueLocation + PathDelim + Lowercase(Name) + '|' + LnC_Settings.ftpSubDir + ftpPathDelim + Lowercase(Name));
            LnC_FM.FMJobs_Add('del|' + QueueLocation + PathDelim + Lowercase(Name) + '|' + 'none');
          end;
      Until FindNext(QueueFiles)<>0;
    end;
  FindClose(QueueFiles);

  //Next upload lws files, create the data structure on ftp site and delete files
  If FindFirst (QueueLocation + PathDelim + '*.lws', faAnyFile - faDirectory, QueueFiles)=0 then
    begin
      Repeat
        With QueueFiles do
          begin
            //Parse Queue data for Queue command information
            SceneFile := TStringList.Create;
            SceneFile.LoadFromFile(QueueLocation + PathDelim + Name);
            ScnName := Name;
            ScnFirstFrame := '';
            ScnLastFrame := '';
            ScnContent := '';
            Groups := 'default';
            //Check for start and end frame as well as content and group
            for i := 0 to SceneFile.Count-1 do
              begin
                s := SceneFile[i];
                s := Lowercase(s);
                s := Trim(s);
                if s <> '' then
                  if Copy(s, 1, 10) = 'firstframe' then
                    begin
                      ScnFirstFrame := Copy(s, 12, Length(s)-10);
                    end;
                if s <> '' then
                  if Copy(s, 1, 9) = 'lastframe' then
                    begin
                      ScnLastFrame := Copy(s, 11, Length(s)-9)
                    end;
                if s <> '' then
                  if Copy(s, 1, 11) = 'lnc_content' then
                    begin
                      ScnContent := Copy(s, 13, Length(s)-11);
                      ScnContent := Trim(ScnContent);
                      ScnContent := Lowercase(ScnContent);
                      if Copy(ScnContent, Length(ScnContent) - 3, 4) = '.zip' then
                        begin
                          //Change space to _
                          for b:=1 to Length(ScnContent) do
                            if ScnContent[b] = ' ' then ScnContent[b] := '_';
                            SceneFile.Delete(i);
                            SceneFile.Insert(i, 'lnc_content ' + ScnContent);
                            //Write file with this new info
                            SceneFile.SaveToFile(QueueLocation + PathDelim + Name);
                        end;
                    end;
                if s <> '' then
                  if Copy(s, 1, 10) = 'lnc_groups' then
                    begin
                      Groups := Copy(s, 12, Length(s)-10);
                      Groups := Trim(Groups);
                      Groups := Lowercase(Groups);
                    end;
              end;
            SceneFile.Free;

            s := '';
            //If the content dir is a zip file, instert in offsite command txt
            if (Copy(ScnContent, Length(ScnContent) - 3, 4) = '.zip') and (ScnContent <> '') then
              begin
                AssignFile(OnsiteCmdsTXTFile, LnC_Settings.WatchFolder + PathDelim + OffsiteCmdTXT);
                {$I-}
                  Reset(OnsiteCmdsTXTFile);
                {$I+}
                IOR := IOResult;
                if IOR = 2 then
                  begin //Create file if it doesn't exist...
                    Rewrite(OnsiteCmdsTXTFile);
                    Writeln(OnsiteCmdsTXTFile, '');
                    Append(OnsiteCmdsTXTFile);
                    Writeln(OnsiteCmdsTXTFile,'add');
                    Writeln(OnsiteCmdsTXTFile,ScnName);
                    Writeln(OnsiteCmdsTXTFile,ScnContent);
                    Writeln(OnsiteCmdsTXTFile,ScnFirstFrame);
                    Writeln(OnsiteCmdsTXTFile,ScnLastFrame);
                    Writeln(OnsiteCmdsTXTFile,Groups);
                  end
                else
                  begin //Append if it does
                    Append(OnsiteCmdsTXTFile);
                    Writeln(OnsiteCmdsTXTFile,'add');
                    Writeln(OnsiteCmdsTXTFile,ScnName);
                    Writeln(OnsiteCmdsTXTFile, ScnContent);
                    Writeln(OnsiteCmdsTXTFile,ScnFirstFrame);
                    Writeln(OnsiteCmdsTXTFile,ScnLastFrame);
                    Writeln(OnsiteCmdsTXTFile,Groups);
                end;
                CloseFile(OnsiteCmdsTXTFile);
              end;
            //If the content dir is not a zip file, instert in onsite command txt
            if (Copy(ScnContent, Length(ScnContent) - 3, 4) <> '.zip') and (ScnContent <> '') then
              begin
                AssignFile(OnsiteCmdsTXTFile, LnC_Settings.WatchFolder + PathDelim + OnsiteCmdTXT);
                {$I-}
                  Reset(OnsiteCmdsTXTFile);
                {$I+}
                IOR := IOResult;
                if IOR = 2 then
                  begin //Create file if it doesn't exist...
                    Rewrite(OnsiteCmdsTXTFile);
                    Writeln(OnsiteCmdsTXTFile, '');
                    Append(OnsiteCmdsTXTFile);
                    Writeln(OnsiteCmdsTXTFile,'add');
                    Writeln(OnsiteCmdsTXTFile,ScnName);
                    Writeln(OnsiteCmdsTXTFile,ScnContent);
                    Writeln(OnsiteCmdsTXTFile,ScnFirstFrame);
                    Writeln(OnsiteCmdsTXTFile,ScnLastFrame);
                    Writeln(OnsiteCmdsTXTFile,Groups);
                  end
                else
                  begin //Append if it does
                    Append(OnsiteCmdsTXTFile);
                    Writeln(OnsiteCmdsTXTFile,'add');
                    Writeln(OnsiteCmdsTXTFile,ScnName);
                    Writeln(OnsiteCmdsTXTFile,ScnContent);
                    Writeln(OnsiteCmdsTXTFile,ScnFirstFrame);
                    Writeln(OnsiteCmdsTXTFile,ScnLastFrame);
                    Writeln(OnsiteCmdsTXTFile,Groups);
                end;
                CloseFile(OnsiteCmdsTXTFile);
              end;
            LnC_FM.FMJobs_Add('put|' + QueueLocation + PathDelim + Lowercase(Name) + '|' + LnC_Settings.ftpSubDir + ftpPathDelim + Lowercase(Name));
            //Creat the output dirs
            ftpOutputdir := LnC_Settings.ftpSubDir + ftpPathDelim + LnC_GetftpOutputDir(LowerCase(Name), 'root');
            ftpOutputdirRGB := LnC_Settings.ftpSubDir + ftpPathDelim + LnC_GetftpOutputDir(LowerCase(Name), 'rgb');
            ftpOutputdirAlpha := LnC_Settings.ftpSubDir + ftpPathDelim + LnC_GetftpOutputDir(LowerCase(Name), 'alpha');
            //Add make dirs to FTP queue
            LnC_FM.FMJobs_Add('mkdir|' + ftpOutputdir + '|' + ftpOutputdir);
            LnC_FM.FMJobs_Add('mkdir|' + ftpOutputdirRGB + '|' + ftpOutputdirRGB);
            LnC_FM.FMJobs_Add('mkdir|' + ftpOutputdirAlpha + '|' + ftpOutputdirAlpha);
            LnC_FM.FMJobs_Add('del|' + QueueLocation + PathDelim + Lowercase(Name) + '|' + 'none');
          end;
      Until FindNext(QueueFiles)<>0;
    end;
  FindClose(QueueFiles);
end;

procedure TLnCWatch.UpdateEverything;

var
  NewINI : TINIFile;
  RowCounter, INICntr, fdiff, ETA, ETAFrameAvrg, AverageFrame, QueueCount, LastQueueCount: integer;
  s, INICntrS, SceneName, ContentDir, StartFrame, EndFrame, Groups:string;
  CurrentTime, LastTime, TimeDiff:LongWord;
  CurrentQueue, QueueLine, LastQueueLine: TStringList;
  ETAmsg, EtaH, EtaM, EtaS:string;
  KeepChecking, SceneFound: Boolean;

begin
  IsUpdating := True;
  //Be sure the system is set to manager the server
  if ManageServer.Checked then
    begin
      //Create blank ini files if they don't exist
      if not FileExistsUTF8(RootDir + PathDelim + OnsiteSceneINI) then
        begin
          NewINI := TIniFile.Create(RootDir + PathDelim + OnsiteSceneINI);
          NewINI.Free;
        end;
      if not FileExistsUTF8(RootDir + PathDelim + OffsiteSceneINI) then
        begin
          NewINI := TIniFile.Create(RootDir + PathDelim + OffsiteSceneINI);
          NewINI.Free;
        end;
      //Rename old ini file to old name
      RenameFileUTF8(RootDir + PathDelim + OnsiteSceneINI, RootDir + PathDelim + OldOnsiteSceneINI);
      RenameFileUTF8(RootDir + PathDelim + OffsiteSceneINI, RootDir + PathDelim + OldOffsiteSceneINI);
      //Create new ini files
      CreateNewINIFiles(RootDir + PathDelim + OldOnsiteSceneINI, RootDir + PathDelim + OnsiteSceneINI, LnC_Settings.WatchFolder + PathDelim + OnsiteCmdTXT);
      CreateNewINIFiles(RootDir + PathDelim + OldOffsiteSceneINI, RootDir + PathDelim + OffsiteSceneINI, LnC_Settings.WatchFolder + PathDelim + OffsiteCmdTXT);
    end;
  //Now Update the interface
  DateSeparator := '-';
  TimeSeparator := ':';
  Label_LastCheck.Caption := 'Last Checked: ' + DateTimeToStr(Now);
  //Reset the Grid
  RowCounter := 0;
  Grid_Scene.Clear;
  Grid_Scene.ColCount := 5;
  Grid_Scene.RowCount := RowCounter + 1;
  Grid_Scene.FixedRows := 1;
  Grid_Scene.Cells[0, RowCounter] := 'Scene';
  Grid_Scene.Cells[1, RowCounter] := 'Content';
  Grid_Scene.Cells[2, RowCounter] := 'Frames';
  Grid_Scene.Cells[3, RowCounter] := 'Render Time Remaining';
  Grid_Scene.Cells[4, RowCounter] := 'Groups';
  CurrentTime := TimeCounter;

  Inc(RowCounter);
  Grid_Scene.RowCount := RowCounter + 1;
  Grid_Scene.Cells[0, RowCounter] := 'Onsite:';

  // Put ini information into CurrentQueue
  //Onsite
  fdiff := 0;
  ETAmsg := '';
  CurrentQueue := TSTringList.Create;
  INICntr := 0;
  StartFrame := '';
  NewINI := TIniFile.Create(RootDir + PathDelim + OnsiteSceneINI);
  while StartFrame <> '[none]'  do
    begin
      Inc(INICntr);
      Str(INICntr, INICntrS);
      StartFrame := NewINI.ReadString(INICntrS, 'StartFrame', '[none]');
      EndFrame := NewINI.ReadString(INICntrS, 'EndFrame', '[none]');
      SceneName := NewINI.ReadString(INICntrS, 'SceneName', '[none]');
      ContentDir := NewINI.ReadString(INICntrS, 'SceneContentDir', '[none]');
      Groups := NewINI.ReadString(INICntrS, 'Groups', 'default');
      AverageFrame := NewINI.ReadInteger(INICntrS, 'AverageFrame', 0);
      if StartFrame <> '[none]' then
        begin
          // SceneName|ContentDir|StartFrame|EndFrame|Groups|AverageFrame|LastTime
          CurrentQueue.Add(SceneName + '|' + ContentDir + '|' + StartFrame + '|' + EndFrame + '|' + Groups + '|' + IntToStr(AverageFrame) + '|' + '0');
        end;
    end;
  if CurrentQueue.Count > 0 then
    begin
      QueueLine := TStringList.Create;
      // Compare current queue to last queue
      for QueueCount := 0 to CurrentQueue.Count - 1 do
        begin
          QueueLine.StrictDelimiter := true;
          QueueLine.Delimiter := '|';
          QueueLine.DelimitedText := CurrentQueue.ValueFromIndex[QueueCount];
          SceneName := QueueLine.ValueFromIndex[0];
          ContentDir := QueueLine.ValueFromIndex[1];
          StartFrame := QueueLine.ValueFromIndex[2];
          EndFrame := QueueLine.ValueFromIndex[3];
          Groups := QueueLine.ValueFromIndex[4];
          AverageFrame := StrToInt(QueueLine.ValueFromIndex[5]);
          LastTime := StrToInt(QueueLine.ValueFromIndex[6]);
          KeepChecking := True;
          LastQueueLine := TStringList.Create;
          LastQueueCount := -1;
          while KeepChecking do
            begin
              Inc(LastQueueCount);
              if LastQueueCount = LastQueueOn.Count then
                KeepChecking := false
              else
                begin
                  LastQueueLine.StrictDelimiter := true;;
                  LastQueueLine.Delimiter := '|';
                  LastQueueLine.DelimitedText := LastQueueOn.ValueFromIndex[LastQueueCount];
                  if (SceneName = LastQueueLine.ValueFromIndex[0]) and
                     (ContentDir = LastQueueLine.ValueFromIndex[1]) and
                     (EndFrame = LastQueueLine.ValueFromIndex[3]) then
                  begin
                    KeepChecking := false;
                  end;
                end;
            end;
          if LastQueueCount < LastQueueOn.Count then
            begin
              // There was a match, need to check for difference
              fdiff := StrToInt(StartFrame) - StrToInt(LastQueueLine.ValueFromIndex[2]);
              if fdiff > 0 then // There was a difference
                begin
                  if StrToInt(LastQueueLine.ValueFromIndex[6]) = 0 then // No time was recorded so this is the first difference
                    begin
                      LastTime := TimeCounter; // Set LastTime to current TimeCounter so it can be used to check difference next check
                      // Update last queue with changes
                      LastQueueOn.Delete(LastQueueCount);
                      LastQueueOn.Insert(LastQueueCount, SceneName + '|' + ContentDir + '|' + StartFrame + '|' + EndFrame + '|' + Groups + '|' + IntToStr(AverageFrame) + '|' + IntToStr(LastTime));
                    end
                  else
                    if fdiff > 4 then // Get at least 5 frames
                      begin
                        // The LastQueue had a time, so we can calculate the differnce
                        TimeDiff := TimeCounter - StrToInt(LastQueueLine.ValueFromIndex[6]);
                        ETAFrameAvrg := ((TimeDiff div fdiff) + AverageFrame) div 2; // Average seconds per frame calculation
                        AverageFrame := ETAFrameAvrg;
                        if AverageFrame < 1 then AverageFrame := 1;
                        LastTime := TimeCounter; // Set LastTime to current TimeCounter so it can be used to check difference next check
                        // Update last queue with changes
                        LastQueueOn.Delete(LastQueueCount);
                        LastQueueOn.Insert(LastQueueCount, SceneName + '|' + ContentDir + '|' + StartFrame + '|' + EndFrame + '|' + Groups + '|' + IntToStr(AverageFrame) + '|' + IntToStr(LastTime));
                      end;
                end;
            end
          else
            begin
              // There was not a match
              LastQueueOn.Add(SceneName + '|' + ContentDir + '|' + StartFrame + '|' + EndFrame + '|' + Groups + '|' + IntToStr(AverageFrame) + '|' + IntToStr(LastTime));
            end;

          // Rebuild Current Queue with any changes
          CurrentQueue.Delete(QueueCount);
          CurrentQueue.Insert(QueueCount, SceneName + '|' + ContentDir + '|' + StartFrame + '|' + EndFrame + '|' + Groups + '|' + IntToStr(AverageFrame) + '|' + IntToStr(LastTime));
          // Update grid
          Inc(RowCounter);
          Grid_Scene.RowCount := RowCounter + 1;
          Grid_Scene.Cells[0, RowCounter] := SceneName;
          Grid_Scene.Cells[1, RowCounter] := ContentDir;
          Grid_Scene.Cells[2, RowCounter] := StartFrame + ' of ' + EndFrame;
          Grid_Scene.Cells[4, RowCounter] := Groups;
          ETAmsg := 'Unknown';
          if AverageFrame > 0 then
            begin
              ETA := (StrToInt(EndFrame) - StrToInt(StartFrame)) * AverageFrame; //Reuse fInt1 to get total seconds left in Queue
              EtaH := IntToStr(ETA div 3600); //Hours
              if Length(EtaH) = 1 then EtaH := '0' + EtaH;
              EtaM := IntToStr((ETA mod 3600) div 60); //Minutes
              if Length(EtaM) = 1 then EtaM := '0' + EtaM;
              ETAmsg := EtaH + ':' + EtaM + ' / ' + IntToStr(AverageFrame) + ' second per frame';
            end;
          Grid_Scene.Cells[3, RowCounter] := ETAmsg;
          // Update ini file with new Average frame
          QueueLine.Clear;
          NewINI.WriteInteger(IntToStr(QueueCount + 1), 'AverageFrame', AverageFrame);
        end;
     QueueLine.Free;
     LastQueueLine.Free;
   end
  else
    begin
      LastQueueOn.Clear; // If CurrentQueue is empty, then clear LastQueue
      LastQueueOn.Add('0|0|0|0|0|0|0');
    end;

  // Eliminate OldQueue items that are not in current queue
  LastQueueLine := TStringList.Create;
  for LastQueueCount := LastQueueOn.Count -1 downto 0 do
    begin
      SceneFound := false;
      LastQueueLine.StrictDelimiter := true;
      LastQueueLine.Delimiter := '|';
      LastQueueLine.DelimitedText := LastQueueOn.ValueFromIndex[LastQueueCount];
      SceneName := LastQueueLine.ValueFromIndex[0];
      ContentDir := LastQueueLine.ValueFromIndex[1];
      EndFrame := LastQueueLine.ValueFromIndex[3];
      QueueLine := TStringList.Create;
      QueueCount := 0;
      while QueueCount < CurrentQueue.Count do
        begin
          QueueLine.StrictDelimiter := true;;
          QueueLine.Delimiter := '|';
          QueueLine.DelimitedText := CurrentQueue.ValueFromIndex[QueueCount];
          if (SceneName = QueueLine.ValueFromIndex[0]) and
             (ContentDir = QueueLine.ValueFromIndex[1]) and
             (EndFrame = QueueLine.ValueFromIndex[3]) then
          begin
            SceneFound := true;
          end;
          Inc(QueueCount);
        end;
      if not SceneFound then
        LastQueueOn.Delete(LastQueueCount);
      QueueLine.Clear;
    end;
  LastQueueLine.Free;
  QueueLine.Free;

{  for LastQueueCount := LastQueueOn.Count -1 downto 0 do
    begin
      ShowMessage(LastQueueOn.ValueFromIndex[LastQueueCount]);
    end;}
  CurrentQueue.Free;
  NewINI.Free;
  // Remove temp ini file
  if FileExistsUTF8(LnC_Settings.WatchFolder + PathDelim + OnsiteSceneINI) then
    DeleteFileUTF8(LnC_Settings.WatchFolder + PathDelim + OnsiteSceneINI);

  Inc(RowCounter);
  Grid_Scene.RowCount := RowCounter + 1;
  Grid_Scene.Cells[0, RowCounter] := 'Offsite:';

  //Offsite
  fdiff := 0;
  ETAmsg := '';
  CurrentQueue := TSTringList.Create;
  INICntr := 0;
  StartFrame := '';
  NewINI := TIniFile.Create(RootDir + PathDelim + OffsiteSceneINI);
  while StartFrame <> '[none]'  do
    begin
      Inc(INICntr);
      Str(INICntr, INICntrS);
      StartFrame := NewINI.ReadString(INICntrS, 'StartFrame', '[none]');
      EndFrame := NewINI.ReadString(INICntrS, 'EndFrame', '[none]');
      SceneName := NewINI.ReadString(INICntrS, 'SceneName', '[none]');
      ContentDir := NewINI.ReadString(INICntrS, 'SceneContentDir', '[none]');
      Groups := NewINI.ReadString(INICntrS, 'Groups', 'default');
      AverageFrame := NewINI.ReadInteger(INICntrS, 'AverageFrame', 0);
      if StartFrame <> '[none]' then
        begin
          // SceneName|ContentDir|StartFrame|EndFrame|Groups|AverageFrame|LastTime
          CurrentQueue.Add(SceneName + '|' + ContentDir + '|' + StartFrame + '|' + EndFrame + '|' + Groups + '|' + IntToStr(AverageFrame) + '|' + '0');
        end;
    end;
  if CurrentQueue.Count > 0 then
    begin
      QueueLine := TStringList.Create;
      // Compare current queue to last queue
      for QueueCount := 0 to CurrentQueue.Count - 1 do
        begin
          QueueLine.StrictDelimiter := true;
          QueueLine.Delimiter := '|';
          QueueLine.DelimitedText := CurrentQueue.ValueFromIndex[QueueCount];
          SceneName := QueueLine.ValueFromIndex[0];
          ContentDir := QueueLine.ValueFromIndex[1];
          StartFrame := QueueLine.ValueFromIndex[2];
          EndFrame := QueueLine.ValueFromIndex[3];
          Groups := QueueLine.ValueFromIndex[4];
          AverageFrame := StrToInt(QueueLine.ValueFromIndex[5]);
          LastTime := StrToInt(QueueLine.ValueFromIndex[6]);
          KeepChecking := True;
          LastQueueLine := TStringList.Create;
          LastQueueCount := -1;
          while KeepChecking do
            begin
              Inc(LastQueueCount);
              if LastQueueCount = LastQueueOff.Count then
                KeepChecking := false
              else
                begin
                  LastQueueLine.StrictDelimiter := true;;
                  LastQueueLine.Delimiter := '|';
                  LastQueueLine.DelimitedText := LastQueueOff.ValueFromIndex[LastQueueCount];
                  if (SceneName = LastQueueLine.ValueFromIndex[0]) and
                     (ContentDir = LastQueueLine.ValueFromIndex[1]) and
                     (EndFrame = LastQueueLine.ValueFromIndex[3]) then
                  begin
                    KeepChecking := false;
                  end;
                end;
            end;
          if LastQueueCount < LastQueueOff.Count then
            begin
              // There was a match, need to check for difference
              fdiff := StrToInt(StartFrame) - StrToInt(LastQueueLine.ValueFromIndex[2]);
              if fdiff > 0 then // There was a difference
                begin
                  if StrToInt(LastQueueLine.ValueFromIndex[6]) = 0 then // No time was recorded so this is the first difference
                    begin
                      LastTime := TimeCounter; // Set LastTime to current TimeCounter so it can be used to check difference next check
                      // Update last queue with changes
                      LastQueueOff.Delete(LastQueueCount);
                      LastQueueOff.Insert(LastQueueCount, SceneName + '|' + ContentDir + '|' + StartFrame + '|' + EndFrame + '|' + Groups + '|' + IntToStr(AverageFrame) + '|' + IntToStr(LastTime));
                    end
                  else
                    if fdiff > 4 then // Get at least 5 frames
                      begin
                        // The LastQueue had a time, so we can calculate the differnce
                        TimeDiff := TimeCounter - StrToInt(LastQueueLine.ValueFromIndex[6]);
                        ETAFrameAvrg := ((TimeDiff div fdiff) + AverageFrame) div 2; // Average seconds per frame calculation
                        AverageFrame := ETAFrameAvrg;
                        if AverageFrame < 1 then AverageFrame := 1;
                        LastTime := TimeCounter; // Set LastTime to current TimeCounter so it can be used to check difference next check
                        // Update last queue with changes
                        LastQueueOff.Delete(LastQueueCount);
                        LastQueueOff.Insert(LastQueueCount, SceneName + '|' + ContentDir + '|' + StartFrame + '|' + EndFrame + '|' + Groups + '|' + IntToStr(AverageFrame) + '|' + IntToStr(LastTime));
                      end;
                end;
            end
          else
            begin
              // There was not a match
              LastQueueOff.Add(SceneName + '|' + ContentDir + '|' + StartFrame + '|' + EndFrame + '|' + Groups + '|' + IntToStr(AverageFrame) + '|' + IntToStr(LastTime));
            end;

          // Rebuild Current Queue with any changes
          CurrentQueue.Delete(QueueCount);
          CurrentQueue.Insert(QueueCount, SceneName + '|' + ContentDir + '|' + StartFrame + '|' + EndFrame + '|' + Groups + '|' + IntToStr(AverageFrame) + '|' + IntToStr(LastTime));
          // Update grid
          Inc(RowCounter);
          Grid_Scene.RowCount := RowCounter + 1;
          Grid_Scene.Cells[0, RowCounter] := SceneName;
          Grid_Scene.Cells[1, RowCounter] := ContentDir;
          Grid_Scene.Cells[2, RowCounter] := StartFrame + ' of ' + EndFrame;
          Grid_Scene.Cells[4, RowCounter] := Groups;
          ETAmsg := 'Unknown';
          if AverageFrame > 0 then
            begin
              ETA := (StrToInt(EndFrame) - StrToInt(StartFrame)) * AverageFrame; //Reuse fInt1 to get total seconds left in Queue
              EtaH := IntToStr(ETA div 3600); //Hours
              if Length(EtaH) = 1 then EtaH := '0' + EtaH;
              EtaM := IntToStr((ETA mod 3600) div 60); //Minutes
              if Length(EtaM) = 1 then EtaM := '0' + EtaM;
              ETAmsg := EtaH + ':' + EtaM + ' / ' + IntToStr(AverageFrame) + ' second per frame';
            end;
          Grid_Scene.Cells[3, RowCounter] := ETAmsg;
          // Update ini file with new Average frame
          QueueLine.Clear;
          NewINI.WriteInteger(IntToStr(QueueCount + 1), 'AverageFrame', AverageFrame);
        end;
     QueueLine.Free;
     LastQueueLine.Free;
   end
  else
    begin
      LastQueueOff.Clear; // If CurrentQueue is empty, then clear LastQueue
      LastQueueOff.Add('0|0|0|0|0|0|0');
    end;

  // Eliminate OldQueue items that are not in current queue
  LastQueueLine := TStringList.Create;
  for LastQueueCount := LastQueueOff.Count -1 downto 0 do
    begin
      SceneFound := false;
      LastQueueLine.StrictDelimiter := true;
      LastQueueLine.Delimiter := '|';
      LastQueueLine.DelimitedText := LastQueueOff.ValueFromIndex[LastQueueCount];
      SceneName := LastQueueLine.ValueFromIndex[0];
      ContentDir := LastQueueLine.ValueFromIndex[1];
      EndFrame := LastQueueLine.ValueFromIndex[3];
      QueueLine := TStringList.Create;
      QueueCount := 0;
      while QueueCount < CurrentQueue.Count do
        begin
          QueueLine.StrictDelimiter := true;;
          QueueLine.Delimiter := '|';
          QueueLine.DelimitedText := CurrentQueue.ValueFromIndex[QueueCount];
          if (SceneName = QueueLine.ValueFromIndex[0]) and
             (ContentDir = QueueLine.ValueFromIndex[1]) and
             (EndFrame = QueueLine.ValueFromIndex[3]) then
          begin
            SceneFound := true;
          end;
          Inc(QueueCount);
        end;
      if not SceneFound then
        LastQueueOff.Delete(LastQueueCount);
      QueueLine.Clear;
    end;
  LastQueueLine.Free;
  QueueLine.Free;

{  for LastQueueCount := LastQueueOff.Count -1 downto 0 do
    begin
      ShowMessage(LastQueueOff.ValueFromIndex[LastQueueCount]);
    end;}
  CurrentQueue.Free;
  NewINI.Free;
  // Remove temp ini file
  if FileExistsUTF8(LnC_Settings.WatchFolder + PathDelim + OffsiteSceneINI) then
    DeleteFileUTF8(LnC_Settings.WatchFolder + PathDelim + OffsiteSceneINI);

  //Be sure the system is set to manager the server
  if ManageServer.Checked then
    begin
      s := LnC_MakeLastUpdate (RootDir + PathDelim, LnC_Settings.ScreamerName);
      //Send ini files back to the FTP server
      LnC_FM.FMJobs_Add('put|' + RootDir + PathDelim + OnsiteSceneINI + '|' + LnC_Settings.ftpSubDir + ftpPathDelim + OnsiteSceneINI);
      LnC_FM.FMJobs_Add('put|' + RootDir + PathDelim + OffsiteSceneINI + '|' + LnC_Settings.ftpSubDir + ftpPathDelim + OffsiteSceneINI);
      LnC_FM.FMJobs_Add('put|' + s + '|' + LnC_Settings.ftpSubDir + ftpPathDelim + ExtractFilename(s));
    end;
  LnC_FM.FMJobs_Add('del|' + RootDir + PathDelim + OnsiteSceneINI + '|' + 'none');
  LnC_FM.FMJobs_Add('del|' + RootDir + PathDelim + OffsiteSceneINI + '|' + 'none');
  IsUpdating := False;
end;

procedure TLnCWatch.FTP_TimerTimer(Sender: TObject);
var
  FTPMessages: TStringList;
  i: integer;

begin
  //Trigger FTP system
  LnC_FM.Run(LnC_Settings.ftpServer, LnC_Settings.ftpPort, LnC_Settings.ftpUser, LnC_Settings.ftpPassword);
  ftpProgressBar.Position := LnC_FM.Progress;
  ftpLabel1.Caption := LnC_FM.MessageLine1;
  ftpLabel2.Caption := LnC_FM.MessageLine2;

  if (not LnC_FM.IsBusy) and (not IsUpdating) and (not LnC_FM.HasJobsPending) then
    begin
      If (FileExistsUTF8(RootDir + PathDelim + OnsiteSceneINI)) and
         (FileExistsUTF8(RootDir + PathDelim + OffsiteSceneINI)) then
        begin
          UpdateEverything;
        end;
    end;
  if (CloseASAP) and (not LnC_FM.HasJobsPending) then
    begin
      Close;
    end;

    if (Length(LnC_FM.Messages) > 0) and (LnC_FM.Messages <> LineEnding)then
      begin
        FTPMessages := TStringList.Create;
        FTPMessages.Text := LnC_FM.Messages;
        for i := 0 to FTPMessages.Count - 1 do
          if FTPMessages.ValueFromIndex[i] <> '' then
            Memo_FTPmsg.Append(FTPMessages.ValueFromIndex[i]);
        LnC_FM.Messages := '';
        FTPMessages.Free;
      end;
end;

procedure TLnCWatch.InitializeServerClick(Sender: TObject);
var
  NewINI : TINIFile;
  s : string;

begin
  if ManageServer.Checked then //Be sure it is set to manage the server
    begin
      if MessageDlg('Initialize Server',
                    'Would you like to initialize the server with blank configuration files?',
                    mtConfirmation, [mbYes, mbNo],0) = mrYes then
        begin
          // Delete ini files if they exist
          if FileExistsUTF8(RootDir + PathDelim + OnsiteSceneINI) then DeleteFileUTF8(RootDir + PathDelim + OnsiteSceneINI);
          if FileExistsUTF8(RootDir + PathDelim + OffsiteSceneINI) then DeleteFileUTF8(RootDir + PathDelim + OffsiteSceneINI);
          NewINI := TIniFile.Create(RootDir + PathDelim + OnsiteSceneINI);
          NewINI.WriteString('LnC', 'Version', LnC_Settings.VersionString);
          NewINI.Free;
          NewINI := TIniFile.Create(RootDir + PathDelim + OffsiteSceneINI);
          NewINI.WriteString('LnC', 'Version', LnC_Settings.VersionString);
          NewINI.Free;
          s := LnC_MakeLastUpdate (RootDir + PathDelim, LnC_Settings.ScreamerName);
          LnC_FM.ErrorReset;
          //Send ini files back to the FTP server
          LnC_FM.FMJobs_Add('put|' + RootDir + PathDelim + OnsiteSceneINI + '|' + LnC_Settings.ftpSubDir + ftpPathDelim + OnsiteSceneINI);
          LnC_FM.FMJobs_Add('put|' + RootDir + PathDelim + OffsiteSceneINI + '|' + LnC_Settings.ftpSubDir + ftpPathDelim + OffsiteSceneINI);
          LnC_FM.FMJobs_Add('put|' + s + '|' + LnC_Settings.ftpSubDir + ftpPathDelim + ExtractFilename(s));
          LnC_FM.FMJobs_Add('del|' + RootDir + PathDelim + OnsiteSceneINI + '|' + 'none');
          LnC_FM.FMJobs_Add('del|' + RootDir + PathDelim + OffsiteSceneINI + '|' + 'none')
        end;
    end
  else
    ShowMessage('This Watcher is not set to manage the server.');
end;

procedure TLnCWatch.INI_TimerTimer(Sender: TObject);
begin
  //In case of a fatal error, reset so LnC can try again
  LnC_FM.ErrorReset;

  //Be sure the system is set to manage the server
  if ManageServer.Checked then
    begin
      //Segment the Queue, move the files from watch folder to the upload queue and then upload the files
      //Segment scenes and uploadQueue also addes commands to command files
      SegmentScenes(LnC_Settings.WatchFolder + PathDelim);
      MoveWatchedFiles('*.zip');
      UploadQueue(LnC_Settings.WatchFolder + PathDelim + WatcherUploadQueue);
    end;

  //Trigger ini OutputDownload
  LnC_FM.FMJobs_Add('get|' + RootDir + PathDelim + OnsiteSceneINI + '|' + LnC_Settings.ftpSubDir + ftpPathDelim + OnsiteSceneINI);
  LnC_FM.FMJobs_Add('get|' + RootDir + PathDelim + OffsiteSceneINI + '|' + LnC_Settings.ftpSubDir + ftpPathDelim + OffsiteSceneINI);
end;

procedure TLnCWatch.ManageServerClick(Sender: TObject);
begin
  if ManageServer.Checked then
    ManageServer.Checked := false
  else
    ManageServer.Checked := true;
end;

procedure TLnCWatch.OutputDownloadClick(Sender: TObject);
var
  WatchSceneDownload: TWatchSceneDownload;
  i: integer;

begin
  if (LnC_FM.HasJobsPending) then
    MessageDlg('Please wait...', 'LightNet: Watcher is currently working. Please try again in a moment.', mtInformation, [mbOK], 0)
  else
    begin
      WatchSceneDownload := TWatchSceneDownload.Create(self);
      if WatchSceneDownload.ShowModal=mrOk then
        begin
          for i := 0 to WatchSceneDownload.Memo_Listing.Lines.Count - 1 do
            begin
              LnC_FM.FMJobs_Add('get|' + WatchSceneDownload.Memo_Listing.Lines.ValueFromIndex[i]);
            end;
        end;
      WatchSceneDownload.Free;
    end;
end;

procedure TLnCWatch.SceneAddClick(Sender: TObject);
var
  WatchAdd: TWatchAdd;
  IOR: integer;
  f:textfile;

begin
  if (LnC_FM.HasJobsPending) then
    MessageDlg('Please wait...', 'LightNet: Watcher is currently working. Please try again in a moment.', mtInformation, [mbOK], 0)
  else
    begin
      WatchAdd := TWatchAdd.Create(self);
      if WatchAdd.ShowModal=mrOk then
        begin
          //setup file
          if WatchAdd.RadioOnsite.Checked then
            begin
              AssignFile(f, LnC_Settings.WatchFolder + PathDelim + OnsiteCmdTXT);
            end;
          if WatchAdd.RadioOffsite.Checked then
            begin
              AssignFile(f, LnC_Settings.WatchFolder + PathDelim + OffsiteCmdTXT);
            end;
          {$I-}
            Reset(f);
          {$I+}
          IOR := IOResult;
          if IOR = 2 then
            begin //Create file if it doesn't exist...
              Rewrite(f);
            end
          else
            begin //Append if it does
              Append(f);
            end;
          Writeln(f, 'add');
          Writeln(f, WatchAdd.ComboBox_Scene.Items.ValueFromIndex[WatchAdd.ComboBox_Scene.ItemIndex]);
          Writeln(f, trim(WatchAdd.ContentDir.Text));
          Writeln(f, trim(WatchAdd.FirstFrame.Text));
          Writeln(f, trim(WatchAdd.LastFrame.Text));
          Writeln(f, trim(WatchAdd.Groups.Text));
          CloseFile(f);
          INI_TimerTimer(self);
        end;
      WatchAdd.Free;
    end;
end;

procedure TLnCWatch.SceneGapCheckClick(Sender: TObject);
var
  WatchGapCheck: TWatchGapCheck;
  i, IOR: integer;
  f:textfile;

begin
  if (LnC_FM.HasJobsPending) then
    MessageDlg('Please wait...', 'LightNet: Watcher is currently working. Please try again in a moment.', mtInformation, [mbOK], 0)
  else
    begin
      WatchGapCheck := TWatchGapCheck.Create(self);
      if WatchGapCheck.ShowModal=mrOk then
        begin
          //setup file
          if not WatchGapCheck.SceneOffsite then
            begin
              AssignFile(f, LnC_Settings.WatchFolder + PathDelim + OnsiteCmdTXT);
            end;
          if WatchGapCheck.SceneOffsite then
            begin
              AssignFile(f, LnC_Settings.WatchFolder + PathDelim + OffsiteCmdTXT);
            end;
          {$I-}
            Reset(f);
          {$I+}
          IOR := IOResult;
          if IOR = 2 then
            begin //Create file if it doesn't exist...
              Rewrite(f);
            end
          else
            begin //Append if it does
              Append(f);
            end;
          for i := 0 to WatchGapCheck.Memo_Listing.Lines.Count - 1 do
            Writeln(f, WatchGapCheck.Memo_Listing.Lines.ValueFromIndex[i]);
          Writeln(f, 'top');
          Writeln(f, WatchGapCheck.ComboBox_Scene.Items.ValueFromIndex[WatchGapCheck.ComboBox_Scene.ItemIndex]);
          CloseFile(f);
          INI_TimerTimer(self);
        end;
      WatchGapCheck.Free;
    end;
end;

procedure TLnCWatch.ScenePurgeClick(Sender: TObject);
var
  WatchScenePurge: TWatchScenePurge;
  i: integer;

begin
  if (LnC_FM.HasJobsPending) then
    MessageDlg('Please wait...', 'LightNet: Watcher is currently working. Please try again in a moment.', mtInformation, [mbOK], 0)
  else
    begin
      WatchScenePurge := TWatchScenePurge.Create(self);
      if WatchScenePurge.ShowModal=mrOk then
        begin
          for i := 0 to WatchScenePurge.Memo_Listing.Lines.Count - 1 do
            begin
              LnC_FM.FMJobs_Add('ftpdel|' + WatchScenePurge.Memo_Listing.Lines.ValueFromIndex[i]);
            end;
        end;
      WatchScenePurge.Free;
    end;
end;

procedure TLnCWatch.SceneRemoveClick(Sender: TObject);
var
  WatchRemove: TWatchRemove;
  IOR: integer;
  f:textfile;

begin
  if (LnC_FM.HasJobsPending) then
    MessageDlg('Please wait...', 'LightNet: Watcher is currently working. Please try again in a moment.', mtInformation, [mbOK], 0)
  else
    begin
      WatchRemove := TWatchRemove.Create(self);
      if WatchRemove.ShowModal=mrOk then
        begin
          //setup file
          if WatchRemove.RadioOnsite.Checked then
            begin
              AssignFile(f, LnC_Settings.WatchFolder + PathDelim + OnsiteCmdTXT);
            end;
          if WatchRemove.RadioOffsite.Checked then
            begin
              AssignFile(f, LnC_Settings.WatchFolder + PathDelim + OffsiteCmdTXT);
            end;
          {$I-}
            Reset(f);
          {$I+}
          IOR := IOResult;
          if IOR = 2 then
            begin //Create file if it doesn't exist...
              Rewrite(f);
            end
          else
            begin //Append if it does
              Append(f);
            end;
          Writeln(f, 'remove');
          Writeln(f, WatchRemove.ComboBox_Scene.Items.ValueFromIndex[WatchRemove.ComboBox_Scene.ItemIndex]);
          CloseFile(f);
          INI_TimerTimer(self);
        end;
      WatchRemove.Free;
    end;
end;

procedure TLnCWatch.SceneTopClick(Sender: TObject);
var
  WatchTop: TWatchTop;
  IOR: integer;
  f:textfile;

begin
  if (LnC_FM.HasJobsPending) then
    MessageDlg('Please wait...', 'LightNet: Watcher is currently working. Please try again in a moment.', mtInformation, [mbOK], 0)
  else
    begin
      WatchTop := TWatchTop.Create(self);
      if WatchTop.ShowModal=mrOk then
        begin
          //setup file
          if WatchTop.RadioOnsite.Checked then
            begin
              AssignFile(f, LnC_Settings.WatchFolder + PathDelim + OnsiteCmdTXT);
            end;
          if WatchTop.RadioOffsite.Checked then
            begin
              AssignFile(f, LnC_Settings.WatchFolder + PathDelim + OffsiteCmdTXT);
            end;
          {$I-}
            Reset(f);
          {$I+}
          IOR := IOResult;
          if IOR = 2 then
            begin //Create file if it doesn't exist...
              Rewrite(f);
            end
          else
            begin //Append if it does
              Append(f);
            end;
          Writeln(f, 'top');
          Writeln(f, WatchTop.ComboBox_Scene.Items.ValueFromIndex[WatchTop.ComboBox_Scene.ItemIndex]);
          CloseFile(f);
          INI_TimerTimer(self);
        end;
      WatchTop.Free;
    end;
end;

procedure TLnCWatch.SegmentMaker(const SceneFile:TStringList; const i: integer; const RegionLine, ScenePathBase, Suffix: string);

var
  NewSceneFile: TStringList;
  i2: integer;
  s: string;
  ChangeOutput: boolean;

begin
  s:= '';
  ChangeOutput := false;
  NewSceneFile := TStringList.Create;
  for i2 := 0 to SceneFile.Count -1 do
    NewSceneFile.Append(SceneFile.ValueFromIndex[i2]);
  i2 := 0;

  if NewSceneFile.IndexOf('lnc_preserveOutput 2') > 0 then
    ChangeOutput := True
  else
    ChangeOutput := False;

  NewSceneFile.Insert(i, RegionLine);
  NewSceneFile.Insert(i, 'LimitedRegion 2');

  if ChangeOutput then
    begin
      if NewSceneFile.IndexOf('SaveRGB 1') > 0 then
        begin
          i2 := NewSceneFile.IndexOf('SaveRGB 1') + 1;
          s := NewSceneFile[i2] + Suffix;
          NewSceneFile.Delete(i2);
          NewSceneFile.Insert(i2, s);
        end;
      if NewSceneFile.IndexOf('SaveAlpha 1') > 0 then
        begin
          i2 := NewSceneFile.IndexOf('SaveAlpha 1') + 1;
          s := NewSceneFile[i2] + Suffix;
          NewSceneFile.Delete(i2);
          NewSceneFile.Insert(i2, s);
        end;
    end;
  NewSceneFile.SaveToFile(ScenePathBase + Suffix + 'seg.lws');
  NewSceneFile.Free;
end;

procedure TLnCWatch.SegmentScenes(const SceneDir:string);

var
  SceneOriginal :TStringList;
  SceneFiles :TSearchRec;
  SceneNameBase:string;
  SegCount, LRIndex, LRCleanIndex :integer;

  s, s2:string;
  i, e :integer;

begin
  i := 0;
  s := '';
  s2 := '';
  DateSeparator := '-';
  TimeSeparator := '_';

  //Add date stamp
  If FindFirst (SceneDir + '*.lws', faAnyFile - faDirectory, SceneFiles)=0 then
    begin
      Repeat
        With SceneFiles do
          begin
            s := Name;
            s := Copy(s, 1, Length(s) - 4);
            s2 := '(' + DateTimeToStr(Now) + ').lws';
            RenameFileUTF8(SceneDir + PathDelim + Name, SceneDir + PathDelim + s + s2);
          end;
      Until FindNext(SceneFiles)<>0;
    end;
  FindClose(SceneFiles);

  //Reset variables
  s := '';
  i := 0;
  SegCount := 1;
  LRIndex := 0;
  LRCleanIndex := 0;

  If FindFirst (SceneDir + '*.lws', faAnyFile - faDirectory, SceneFiles)=0 then
    begin
      Repeat
        With SceneFiles do
          begin
            SceneOriginal := TStringList.Create;
            SceneOriginal.LoadFromFile(SceneDir + Name);
            SceneNameBase := Copy(Name, 1, Length(Name) - 4);

            //Check for segment number
            for i := 0 to SceneOriginal.Count-1 do
              begin
                s := SceneOriginal[i];
                s := Lowercase(s);
                s := Trim(s);
                if s <> '' then
                  if Copy(s, 1, 11) = 'lnc_segment' then
                    begin
                      Val(Copy(s, 13, Length(s)-11), SegCount, e);
                    end;
              end;


            //RegionLimits format: X X Y Y
            case SegCount of
            1:begin
                //do nothing
              end;
            2:begin
                //4x Segmentation
                // Find the location for the limit reigon command and
                //any previous LR section
                for i := 0 to SceneOriginal.Count-1 do
                  begin
                    s := SceneOriginal[i];
                    if s <> '' then
                      if Copy(s, 1, 17) = 'GlobalPixelAspect' then
                        begin
                          LRIndex := i + 1;
                        end;
                    if s <> '' then
                      if Copy(s, 1, 13) = 'LimitedRegion' then
                        begin
                          LRCleanIndex := i;
                        end;
                  end;
                //Eliminate any current limit reigon settings
                if LRCleanIndex > 0 then
                  begin
                    SceneOriginal.Delete(LRCleanIndex);
                    SceneOriginal.Delete(LRCleanIndex);
                  end;
                //A1
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0 0.5 0 0.5', SceneDir + SceneNameBase, '_a1_');
                //A2
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0.5 1 0 0.5', SceneDir + SceneNameBase, '_a2_');
                //B1
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0 0.5 0.5 1', SceneDir + SceneNameBase, '_b1_');
                //B2
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0.5 1 0.5 1', SceneDir + SceneNameBase, '_b2_');
                DeleteFileUTF8(SceneDir + Name);
              end;
            3:begin
                //16x Segmentation
                // Find the location for the limit reigon command and
                //any previous LR section
                for i := 0 to SceneOriginal.Count-1 do
                  begin
                    s := SceneOriginal[i];
                    if s <> '' then
                      if Copy(s, 1, 17) = 'GlobalPixelAspect' then
                        begin
                          LRIndex := i + 1;
                        end;
                    if s <> '' then
                      if Copy(s, 1, 13) = 'LimitedRegion' then
                        begin
                          LRCleanIndex := i;
                        end;
                  end;
                //Eliminate any current limit reigon settings
                if LRCleanIndex > 0 then
                  begin
                    SceneOriginal.Delete(LRCleanIndex);
                    SceneOriginal.Delete(LRCleanIndex);
                  end;
                //A1
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0 0.25 0 0.25', SceneDir + SceneNameBase, '_a1_');
                //A2
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0.25 0.5 0 0.25', SceneDir + SceneNameBase, '_a2_');
                //A3
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0.5 0.75 0 0.25', SceneDir + SceneNameBase, '_a3_');
                //A4
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0.75 1 0 0.25', SceneDir + SceneNameBase, '_a4_');
                //B1
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0 0.25 0.25 0.5', SceneDir + SceneNameBase, '_b1_');
                //B2
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0.25 0.5 0.25 0.5', SceneDir + SceneNameBase, '_b2_');
                //B3
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0.5 0.75 0.25 0.5', SceneDir + SceneNameBase, '_b3_');
                //B4
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0.75 1 0.25 0.5', SceneDir + SceneNameBase, '_b4_');
                //C1
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0 0.25 0.5 0.75', SceneDir + SceneNameBase, '_c1_');
                //C2
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0.25 0.5 0.5 0.75', SceneDir + SceneNameBase, '_c2_');
                //C3
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0.5 0.75 0.5 0.75', SceneDir + SceneNameBase, '_c3_');
                //C4
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0.75 1 0.5 0.75', SceneDir + SceneNameBase, '_c4_');
                //D1
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0 0.25 0.75 1', SceneDir + SceneNameBase, '_d1_');
                //D2
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0.25 0.5 0.75 1', SceneDir + SceneNameBase, '_d2_');
                //D3
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0.5 0.75 0.75 1', SceneDir + SceneNameBase, '_d3_');
                //D4
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0.75 1 0.75 1', SceneDir + SceneNameBase, '_d4_');
                DeleteFileUTF8(SceneDir + Name);
              end;
            4:begin
                //25x Segmentation
                // Find the location for the limit reigon command and
                //any previous LR section
                for i := 0 to SceneOriginal.Count-1 do
                  begin
                    s := SceneOriginal[i];
                    if s <> '' then
                      if Copy(s, 1, 17) = 'GlobalPixelAspect' then
                        begin
                          LRIndex := i + 1;
                        end;
                    if s <> '' then
                      if Copy(s, 1, 13) = 'LimitedRegion' then
                        begin
                          LRCleanIndex := i;
                        end;
                  end;
                //Eliminate any current limit reigon settings
                if LRCleanIndex > 0 then
                  begin
                    SceneOriginal.Delete(LRCleanIndex);
                    SceneOriginal.Delete(LRCleanIndex);
                  end;
                //A1
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0 0.2 0 0.2', SceneDir + SceneNameBase, '_a1_');
                //A2
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0.2 0.4 0 0.2', SceneDir + SceneNameBase, '_a2_');
                //A3
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0.4 0.6 0 0.2', SceneDir + SceneNameBase, '_a3_');
                //A4
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0.6 .8 0 0.2', SceneDir + SceneNameBase, '_a4_');
                //A5
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0.8 1 0 0.2', SceneDir + SceneNameBase, '_a5_');
                //B1
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0 0.2 0.2 0.4', SceneDir + SceneNameBase, '_b1_');
                //B2
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0.2 0.4 0.2 0.4', SceneDir + SceneNameBase, '_b2_');
                //B3
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0.4 0.6 0.2 0.4', SceneDir + SceneNameBase, '_b3_');
                //B4
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0.6 .8 0.2 0.4', SceneDir + SceneNameBase, '_b4_');
                //B5
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0.8 1 0.2 0.4', SceneDir + SceneNameBase, '_b5_');
                //C1
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0 0.2 0.4 0.6', SceneDir + SceneNameBase, '_c1_');
                //C2
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0.2 0.4 0.4 0.6', SceneDir + SceneNameBase, '_c2_');
                //C3
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0.4 0.6 0.4 0.6', SceneDir + SceneNameBase, '_c3_');
                //C4
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0.6 .8 0.4 0.6', SceneDir + SceneNameBase, '_c4_');
                //C5
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0.8 1 0.4 0.6', SceneDir + SceneNameBase, '_c5_');
                //D1
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0 0.2 0.6 0.8', SceneDir + SceneNameBase, '_d1_');
                //D2
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0.2 0.4 0.6 0.8', SceneDir + SceneNameBase, '_d2_');
                //D3
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0.4 0.6 0.6 0.8', SceneDir + SceneNameBase, '_d3_');
                //D4
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0.6 .8 0.6 0.8', SceneDir + SceneNameBase, '_d4_');
                //D5
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0.8 1 0.6 0.8', SceneDir + SceneNameBase, '_d5_');

                //E1
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0 0.2 0.8 1', SceneDir + SceneNameBase, '_e1_');
                //E2
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0.2 0.4 0.8 1', SceneDir + SceneNameBase, '_e2_');
                //E3
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0.4 0.6 0.8 1', SceneDir + SceneNameBase, '_e3_');
                //E4
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0.6 .8 0.8 1', SceneDir + SceneNameBase, '_e4_');
                //E5
                SegmentMaker(SceneOriginal, LRIndex, 'RegionLimits 0.8 1 0.8 1', SceneDir + SceneNameBase, '_e5_');
                DeleteFileUTF8(SceneDir + Name);
              end;
          end;
          SceneOriginal.Free;
        end;
      Until FindNext(SceneFiles)<>0;
    end;
    FindClose(SceneFiles);
    MoveWatchedFiles('*.lws');
end;

initialization
  {$I lncwatcher_form.lrs}

end.

