{
  Copyright 2011-2012 Joe Justice (joe@joejustice.org)
 
  This file is part of LightNet: Cloud.

  LightNet: Cloud is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  LightNet: Cloud is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with LightNet: Cloud, see files COPYING.txt and COPYING.LESSER.txt 
  If not, see <http://www.gnu.org/licenses/>.

----------------------------------------------------------------------------

Handles LightNet: Cloud's settings, including loading and saving them.

}

unit settings_unit;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, lnc_utils;

type
  TLnC_Settings  = class(TObject)
    ScreamerName, LocalWorkingDir, LocalCache, Groups: string;

    Onsite, Offsite, AdaptiveFrameLoading, ScreamerSilent : boolean;

    AutoStartInterval: integer; //Autostart variables
    Autostart: Boolean; //Time in sDisconnectedeconds

    ftpServer, ftpPort, ftpSubDir, ftpUser, ftpPassword: string; //Database settings
    ftpCheckInterval: integer; //Time in seconds

    LWScreamerFile, LWScreamerConfigDir, LWScreamerConfigSuffix: string; //Lightwave Screamer setting
    LWScreamerNumber: integer;

    LnC_BaseSettingsFilename: string;

    //Watcher settings
    WatchManageServer, WatchSilent :boolean;
    WatchFolder:string;
    WatchInterval:integer;

    constructor Create;
    destructor Destroy; override;

    function Save (SettingsFile: String): boolean;
    function Load (SettingsFile: String): boolean;
    function CleanPathStr (pathstr:string): string;
    procedure SetToDefault;

    function VersionString: string;
  end; // End of TSettings Class

implementation

uses
  IniFiles;

const
  //Set variable names for the settings to be saved in the conf file
  confLnCSettingsSection   ='Lightnet: Cloud - Settings';

  confScreamerName         ='ScreamerName';
  confLocalWorkingDir      ='LWScreamerCommandDir';
  confLocalCache           ='LocalCache';
  confGroups               = 'Groups';

  confAutoStartInterval    ='AutoStartInterval';
  confAutostart            ='AutoStart';
  confScreamerSilent       ='ScreamerSilent';

  confOnsite               ='Onsite';
  confOffsite              ='Offsite';
  confAdaptiveFrameLoading ='AdaptiveFrameLoading';

  confftpServer            ='ftpServer';
  confftpPort              ='ftpPort';
  confftpSubDir            ='ftpSubDirectory';
  confftpUser              ='ftpUser';
  confftpPassword          ='ftpPassword';
  confftpCheckInterval     ='ftpCheckInterval';

  confLWScreamerFile       ='LWScreamerFile';
  confLWScreamerConfigDir  ='LWScreamerConfigDir';
  confLWScreamerConfigSuffix='LWScreamerConfigSuffix';
  confLWScreamerNumber     ='LWScreamerNumber';

  confWatchManageServer    ='WatchManageServer';
  confWatchSilent          ='WatchSilent';
  confWatchFolder          ='WatchFolder';
  confWatchInterval        ='WatchInterval';

function EncryptString (const s:string): string;
var
  i:integer;

begin
  result := '';
  for i := 1 to Length(s) do
    result := result + chr(ord(s[i]) xor 21);
end;

function TLnC_Settings.VersionString: string;
  begin
    Result:= 'v' + IntToStr(LnC_VersionMain) + '.' + IntToStr(LnC_VersionSub) + '.' + IntToStr(LnC_VersionSubSub) + ' ' + LnC_VersionName;
  end;

constructor TLnC_Settings.Create;
  begin
    inherited Create;
    LnC_BaseSettingsFilename := 'lightnet_cloud_1_options' + LnC_SettingsFilenameExtention;
    if not Load(LnC_BaseSettingsFilename) then SetToDefault;
    LocalWorkingDir := '';
  end;

destructor TLnC_Settings.Destroy;
  begin
    inherited Destroy;
  end;

procedure TLnC_Settings.SetToDefault;

  begin
    ScreamerName := LnC_NoSetting;
    LocalWorkingDir := LnC_NoSetting;

    Onsite := false;
    Offsite := true;
    AdaptiveFrameLoading := true;

    ftpServer := LnC_NoSetting;
    ftpPort := '21';
    ftpSubDir := LnC_NoSetting;
    ftpUser := LnC_NoSetting;
    ftpPassword := '';
    ftpCheckInterval := 300;

    AutoStartInterval:= 600;
    Autostart:= false;

    LWScreamerFile := LnC_NoSetting;
    LWScreamerConfigDir := LnC_NoSetting;
    LWScreamerConfigSuffix := LnC_NoSetting;
    LWScreamerNumber := 1;
    { Do not intend on giving the user the ability to change
      this LWScreamerNumber. Should always be 1. Only including
      it in case I need to do something with it later. }

    WatchManageServer := false;
    WatchFolder := LnC_NoSetting;
    WatchInterval := 300;
  end;

function TLnC_Settings.Save (SettingsFile: string): boolean;
  var
    F: TIniFile;

  begin
    F:= TIniFile.Create(SettingsFile);

    F.WriteString(confLnCSettingsSection, confScreamerName, ScreamerName);
    F.WriteString(confLnCSettingsSection, confLocalCache, LocalCache);
    F.WriteString(confLnCSettingsSection, confGroups, Groups);

    F.WriteBool(confLnCSettingsSection, confOnsite, Onsite);
    F.WriteBool(confLnCSettingsSection, confOffsite, Offsite);
    F.WriteBool(confLnCSettingsSection, confAdaptiveFrameLoading, AdaptiveFrameLoading);
    F.WriteBool(confLnCSettingsSection, confScreamerSilent, ScreamerSilent);

    F.WriteString(confLnCSettingsSection, confftpServer, ftpServer);
    F.WriteString(confLnCSettingsSection, confftpPort, ftpPort);
    F.WriteString(confLnCSettingsSection, confftpSubDir, ftpSubDir);
    F.WriteString(confLnCSettingsSection, confftpUser, ftpUser);
    F.WriteString(confLnCSettingsSection, confftpPassword, EncryptString(ftpPassword));
    F.WriteInteger(confLnCSettingsSection, confftpCheckInterval, ftpCheckInterval);

    F.WriteInteger(confLnCSettingsSection, confAutostartInterval, AutostartInterval);
    F.WriteBool(confLnCSettingsSection, confAutostart, Autostart);

    F.WriteString(confLnCSettingsSection, confLWScreamerFile, LWScreamerFile);
    F.WriteString(confLnCSettingsSection, confLWScreamerConfigDir, LWScreamerConfigDir);
    F.WriteString(confLnCSettingsSection, confLWScreamerConfigSuffix, LWScreamerConfigSuffix);

    F.WriteBool(confLnCSettingsSection, confWatchManageServer, WatchManageServer);
    F.WriteBool(confLnCSettingsSection, confWatchSilent, WatchSilent);
    F.WriteString(confLnCSettingsSection, confWatchFolder, WatchFolder);
    F.WriteInteger(confLnCSettingsSection, confWatchInterval, WatchInterval);

    Result:= true; //Currently only returns true. May program in failsafes later
    F.Free;
  end;

function TLnC_Settings.Load (SettingsFile: String): boolean;
  var
    F: TIniFile;

  begin
    F:= TIniFile.Create(SettingsFile);
    ScreamerName:= F.ReadString(confLnCSettingsSection, confScreamerName, '');
    LocalCache := F.ReadString(confLnCSettingsSection, confLocalCache, '[none]');
    Groups := F.ReadString(confLnCSettingsSection, confGroups, 'default');

    Onsite:= F.ReadBool(confLnCSettingsSection, confOnsite, false);
    Offsite:= F.ReadBool(confLnCSettingsSection, confOffsite, false);
    AdaptiveFrameLoading:= F.ReadBool(confLnCSettingsSection, confAdaptiveFrameLoading, true);
    ScreamerSilent:= F.ReadBool(confLnCSettingsSection, confScreamerSilent, false);

    ftpServer:= F.ReadString(confLnCSettingsSection, confftpServer, '');
    ftpPort:= F.ReadString(confLnCSettingsSection, confftpPort, '');
    ftpSubDir:= F.ReadString(confLnCSettingsSection, confftpSubDir, '');
    ftpUser:= F.ReadString(confLnCSettingsSection, confftpUser, '');
    ftpPassword:= EncryptString(F.ReadString(confLnCSettingsSection, confftpPassword, ''));
    ftpCheckInterval:= F.ReadInteger(confLnCSettingsSection, confftpCheckInterval, 300);

    AutostartInterval:= F.ReadInteger(confLnCSettingsSection, confAutostartInterval, 600);
    Autostart:= F.ReadBool(confLnCSettingsSection, confAutostart, false);

    LWScreamerFile:= F.ReadString(confLnCSettingsSection, confLWScreamerFile, '');
    LWScreamerConfigDir:= F.ReadString(confLnCSettingsSection, confLWScreamerConfigDir, '');
    LWScreamerConfigSuffix := F.ReadString(confLnCSettingsSection, confLWScreamerConfigSuffix, '');

    WatchManageServer := F.ReadBool(confLnCSettingsSection, confWatchManageServer, false);
    WatchSilent := F.ReadBool(confLnCSettingsSection, confWatchSilent, false);
    WatchFolder := F.ReadString(confLnCSettingsSection, confWatchFolder, '');
    WatchInterval := F.ReadInteger(confLnCSettingsSection, confWatchInterval, 300);

   if F.SectionExists(confLnCSettingsSection) then Result:= true //conf file loaded properly
     else Result:= false; //conf file did not load properly
    F.Free;
  end;

function TLnC_Settings.CleanPathStr (pathstr:string): string;
  var
    s:string;
    i:integer;

  begin
    // Take out spaces and take off trailing / or \ if exists
    s := Trim(pathstr);
    if s <> '' then
      begin
        i := Length(s);
        if s[i] = '\' then
          Delete (s,i, 1);
        if s[i] = '/' then
          Delete (s,i, 1);
      end;

    Result := s;
  end;


end.
