{
  Copyright 2011-2012 Joe Justice (joe@joejustice.org)
 
  This file is part of LightNet: Cloud.

  LightNet: Cloud is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  LightNet: Cloud is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with LightNet: Cloud, see files COPYING.txt and COPYING.LESSER.txt 
  If not, see <http://www.gnu.org/licenses/>.

----------------------------------------------------------------------------

This is the form for adding a scene to the queue in Watcher

}

unit watcher_add_form;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  StdCtrls, watcher_filelister_form, watcher_getsceneinfo_form, lnc_utils;

type

  { TForm1 }

  { TWatchAdd }

  TWatchAdd = class(TForm)
    Groups: TEdit;
    GetInfo: TButton;
    ComboBox_Scene: TComboBox;
    Label_Groups: TLabel;
    Label_LastFrame: TLabel;
    Label_ContentDir: TLabel;
    Label_FirstFrame: TLabel;
    Label_SelectScene: TLabel;
    OK: TButton;
    Cancel: TButton;
    RadioOnsite: TRadioButton;
    RadioOffsite: TRadioButton;
    ContentDir: TEdit;
    FirstFrame: TEdit;
    LastFrame: TEdit;
    procedure FormCreate(Sender: TObject);
    procedure GetInfoClick(Sender: TObject);
    procedure OKClick(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
  end; 

implementation
  uses lncwatcher_form;
{ TWatchAdd }

procedure TWatchAdd.FormCreate(Sender: TObject);
var
  WatchFileLister: TWatchFileLister;
  FileList, LineList: TStringList;
  i: integer;

begin
  WatchFileLister := TWatchFileLister.Create(self);
  WatchFileLister.GetListing(LnC_Settings.ftpSubDir + ftpPathDelim + '*.lws');
  if WatchFileLister.ShowModal = mrOK then
    begin
      FileList := TStringList.Create;
      FileList.Text := WatchFileLister.Listing;
      for i := 0 to FileList.Count - 1 do
        begin
          LineList := TStringList.Create;
          LineList.DelimitedText := FileList.ValueFromIndex[i];
          ComboBox_Scene.Items.Add(LnC_ftpExtractFileName(LineList.ValueFromIndex[8]));
          LineList.Free;
        end;
      FileList.Free;
    end;
  if ComboBox_Scene.Items.Count = 0 then
    ComboBox_Scene.Items.Insert(0, 'No scenes found.')
  else
    GetInfo.Enabled := true;
  ComboBox_Scene.ItemIndex := 0;
  WatchFileLister.Free;
end;

procedure TWatchAdd.GetInfoClick(Sender: TObject);
var
  WatchSceneInfo: TWatchSceneInfo;

begin
  WatchSceneInfo := TWatchSceneInfo.Create(self);
  WatchSceneInfo.GetSceneInfo(LnCWatch.RootDir + PathDelim + ComboBox_Scene.Items.ValueFromIndex[ComboBox_Scene.ItemIndex], LnC_Settings.ftpSubDir + ftpPathDelim + ComboBox_Scene.Items.ValueFromIndex[ComboBox_Scene.ItemIndex]);
  if WatchSceneInfo.ShowModal = mrOK then
    begin
      ContentDir.Text := WatchSceneInfo.ContentDir;
      Groups.Text := WatchSceneInfo.Groups;
      FirstFrame.Text := WatchSceneInfo.FirstFrame;
      LastFrame.Text := WatchSceneInfo.LastFrame;
      if WatchSceneInfo.OffSite then
        RadioOffsite.Checked := true
      else
        RadioOnsite.Checked := true;

    end;
  WatchSceneInfo.Free;
end;

procedure TWatchAdd.OKClick(Sender: TObject);
var
  AllOK: boolean;
begin
  if ComboBox_Scene.Items.ValueFromIndex[0] = 'No scenes found.' then
    ModalResult := mrCancel
  else
    begin
      AllOK := true;
      try
        StrToInt(FirstFrame.Text);
      except
        On E : EConvertError do
          begin
            AllOK := false;
            MessageDlg('Error','First Frame has to be a number.', mtConfirmation, [mbOK], 0)
          end;
      end;
      try
        StrToInt(LastFrame.Text);
      except
        On E : EConvertError do
          begin
            AllOK := false;
            MessageDlg('Error','Last Frame has to be a number.', mtConfirmation, [mbOK], 0)
          end;
      end;
      if AllOK then ModalResult := mrOK;
    end;
end;

initialization
  {$I watcher_add_form.lrs}

end.

