{
  Copyright 2011-2012 Joe Justice (joe@joejustice.org)

  This file is part of LightNet: Cloud.

  LightNet: Cloud is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  LightNet: Cloud is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with LightNet: Cloud, see files COPYING.txt and COPYING.LESSER.txt
  If not, see <http://www.gnu.org/licenses/>.

----------------------------------------------------------------------------

LightNet: Watcher frame gap checking system.

}

unit watcher_gapcheck_form;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  StdCtrls, ExtCtrls, watcher_filelister_form, watcher_getsceneinfo_form, lnc_utils;

type
  { TWatchGapCheck }
  TWatchGapCheck = class(TForm)
    Button_Go: TButton;
    ComboBox_Scene: TComboBox;
    Label1: TLabel;
    Label_SelectScene: TLabel;
    Memo_Listing: TMemo;
    OK: TButton;
    Cancel: TButton;
    procedure Button_GoClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure OKClick(Sender: TObject);
  private
    function IsolateNumber (const FullString: string): string;
    { private declarations }
  public
    SceneOffsite: boolean;
    { public declarations }
  end; 

implementation
  uses lncwatcher_form;

{ TWatchGapCheck }

procedure TWatchGapCheck.FormCreate(Sender: TObject);
var
  WatchFileLister: TWatchFileLister;
  FileList, LineList: TStringList;
  i: integer;

begin
  SceneOffsite := false;
  WatchFileLister := TWatchFileLister.Create(self);
  WatchFileLister.GetListing(LnC_Settings.ftpSubDir + ftpPathDelim + '*.lws');
  if WatchFileLister.ShowModal = mrOK then
    begin
      FileList := TStringList.Create;
      FileList.Text := WatchFileLister.Listing;
      for i := 0 to FileList.Count - 1 do
        begin
          LineList := TStringList.Create;
          LineList.DelimitedText := FileList.ValueFromIndex[i];
          ComboBox_Scene.Items.Add(LnC_ftpExtractFileName(LineList.ValueFromIndex[8]));
          LineList.Free;
        end;
      FileList.Free;
    end;
  WatchFileLister.Free;
  if ComboBox_Scene.Items.Count = 0 then
    begin
      ComboBox_Scene.Items.Insert(0, 'No scenes found.');
      OK.Enabled := false;
      Memo_Listing.ReadOnly := true;
    end
  else
    Button_Go.Enabled := true;
  ComboBox_Scene.ItemIndex := 0;
end;

procedure TWatchGapCheck.OKClick(Sender: TObject);
begin
  if (Memo_Listing.Lines.ValueFromIndex[0] = 'No frames found.') or (Memo_Listing.Lines.ValueFromIndex[0] = 'No gaps detected.') then
    ModalResult := mrCancel
  else
    ModalResult := mrOK;
end;

procedure TWatchGapCheck.Button_GoClick(Sender: TObject);

var
  WatchFileLister: TWatchFileLister;
  WatchSceneInfo: TWatchSceneInfo;
  i, i2, sCount, fStart, fEnd :integer;
  s :string;
  KeepChecking, KeepAdding: Boolean;
  TmpList, LineList: TStringList;

begin
  OK.Enabled := false;
  Cancel.Enabled := false;
  Memo_Listing.ReadOnly := false;
  TmpList := TStringList.Create;
  s := '';
  s := ComboBox_Scene.Items.ValueFromIndex[ComboBox_Scene.ItemIndex];
  s := LnC_GetftpOutputDir(s, 'rgb');
  Memo_Listing.Lines.Clear;
  WatchFileLister := TWatchFileLister.Create(self);
  WatchFileLister.GetListing(LnC_Settings.ftpSubDir + ftpPathDelim + s);
  if WatchFileLister.ShowModal = mrOK then
    begin
      Memo_Listing.Lines.Text := WatchFileLister.Listing;
      //Transfer listing over to TmpList
      for i := 0 to Memo_Listing.Lines.Count - 1 do
        begin
          TmpList.Append(Memo_Listing.Lines.ValueFromIndex[i]);
        end;
      // Remove empty lines and non-numbered
      for i := TmpList.Count - 1 downto 0 do
        begin
          if TmpList.ValueFromIndex[i] = '' then TmpList.Delete(i);
          if IsolateNumber(TmpList.ValueFromIndex[i]) = '' then TmpList.Delete(i);
        end;
      // Remove all size 0 file
      for i := TmpList.Count - 1 downto 0 do
        begin
          LineList := TStringList.Create;
          LineList.DelimitedText := TmpList.ValueFromIndex[i];
          if LineList.ValueFromIndex[4] = '0' then
            begin
              // In case the last file had a size of 0, append a copy to the end
              if i = TmpList.Count - 1 then
                begin
                  TmpList.Append(TmpList.ValueFromIndex[i]);
                  TmpList.Append(TmpList.ValueFromIndex[i]);
                end;
              TmpList.Delete(i);
            end;
          LineList.Free;
        end;
      // Replace string with just number
      for i := 0 to TmpList.Count - 1 do
        begin
          s := IsolateNumber(TmpList.ValueFromIndex[i]);
          TmpList.Delete(i);
          TmpList.Insert(i, s);
        end;
      // Zreo pad and sort
      for i := 0 to TmpList.Count - 1 do
        begin
          s := TmpList.ValueFromIndex[i];
          for i2 := 1 to 9 - Length(TmpList.ValueFromIndex[i]) do
            s := '0' + s;
          TmpList.Delete(i);
          TmpList.Insert(i, s);
        end;
      TmpList.Sort;
      // Replace string with just number
      for i := 0 to TmpList.Count - 1 do
        begin
          s := IsolateNumber(TmpList.ValueFromIndex[i]);
          TmpList.Delete(i);
          TmpList.Insert(i, s);
        end;
      // Copy back to Memo for visual user feedback
      Memo_Listing.Lines.Clear;
      for i := 0 to TmpList.Count - 1 do
        begin
          Memo_Listing.Append(TmpList.ValueFromIndex[i]);
        end;
      s := '';
      if TmpList.Count > 1 then
        ShowMessage('Frame Range: ' + TmpList.ValueFromIndex[0] + ' through ' + TmpList.ValueFromIndex[TmpList.Count - 1]);
      // Check each number and see if its in the listing
      if TmpList.Count > 1 then
        begin
          // If the last file size was 0, it was appened so we need to remove it and then readd once the list has been processed
          if TmpList.ValueFromIndex[TmpList.Count - 1] = TmpList.ValueFromIndex[TmpList.Count - 2] then
            begin
              s := TmpList.ValueFromIndex[TmpList.Count - 1];
              TmpList.Delete(TmpList.Count - 1)
            end;
          // Mathematically count from first listed number to last and check for any missing numbers
          fStart := StrToInt(TmpList.ValueFromIndex[0]);
          fEnd := StrToInt(TmpList.ValueFromIndex[TmpList.Count - 1]);
          for i := fStart to fEnd do
            begin
              KeepChecking := true;
              i2 := 0;
              while KeepChecking do
                begin
                  if i2 >= TmpList.Count - 1 then KeepChecking := false;
                  if IntToStr(i) = TmpList.ValueFromIndex[i2] then KeepChecking := false;
                  Inc(i2);
                end;
              if i2 > TmpList.Count - 1 then
                begin
                  TmpList.Insert(0, IntToStr(i));
                end
              else
                begin
                  TmpList.Delete(i2 - 1);
                end;
            end;
          TmpList.Delete(0); // Deletes last digit, which was copied to the top
          TmpList.Delete(TmpList.Count - 1); // Deletes last digit
          if s > '' then TmpList.Insert(0, s); // In case the last digit was size 0, readd it
        end
      else
        begin
          if TmpList.Count = 1 then
            ShowMessage('Only one frame found:  ' + TmpList[0])
          else
            ShowMessage('No frames found.');
          TmpList.Clear;
        end;
      // Tranfer numbers of to memo
      Memo_Listing.Lines.Clear;
      for i := 0 to TmpList.Count - 1 do
        begin
          Memo_Listing.Append(TmpList.ValueFromIndex[i]);
        end;

      if (Memo_Listing.Lines.Count < 1) then
        begin
          if (Memo_Listing.Lines.ValueFromIndex[0] <> 'No frames found.') then
            Memo_Listing.Lines.Insert(0, 'No gaps detected.');
          Memo_Listing.ReadOnly := true;
        end
      else // There were gaps
        begin
          MessageDlg('Info', IntToStr(Memo_Listing.Lines.Count) + ' frames were missing.', mtInformation, [mbOK], 0);
          WatchSceneInfo := TWatchSceneInfo.Create(self);
          WatchSceneInfo.GetSceneInfo(LnCWatch.RootDir + PathDelim + ComboBox_Scene.Items.ValueFromIndex[ComboBox_Scene.ItemIndex], LnC_Settings.ftpSubDir + ftpPathDelim + ComboBox_Scene.Items.ValueFromIndex[ComboBox_Scene.ItemIndex]);
          i := 0;
          KeepAdding := true;
          if WatchSceneInfo.ShowModal = mrOK then
            begin
              Memo_Listing.ReadOnly := false;
              while KeepAdding do
                begin
                  s := Memo_Listing.Lines.ValueFromIndex[i];
                  Memo_Listing.Lines.Insert(i, 'add');
                  Inc(i);
                  Memo_Listing.Lines.Insert(i, ComboBox_Scene.Items.ValueFromIndex[ComboBox_Scene.ItemIndex]);
                  Inc(i);
                  Memo_Listing.Lines.Insert(i, WatchSceneInfo.ContentDir);
                  Inc(i);
                  Memo_Listing.Lines.Insert(i, s);
                  Inc(i);
                  Inc(i);
                  Memo_Listing.Lines.Insert(i, WatchSceneInfo.Groups);
                  Inc(i);
                  if i >= Memo_Listing.Lines.Count then KeepAdding := false;
                end;
              if WatchSceneInfo.OffSite then
                SceneOffsite := true
              else
                SceneOffsite := false;
            end;
          WatchSceneInfo.Free;
        end;
    end
  else
    begin
      Memo_Listing.Lines.Insert(0, 'No frames found.');
      Memo_Listing.ReadOnly := true;
    end;
  OK.Enabled := true;
  Cancel.Enabled := true;
end;

function TWatchGapCheck.IsolateNumber (const FullString: string): string;
var
  s: string;
  sCount: integer;

begin
  // Remove extention
  s := FullString;
  sCount := Length(s);
  while (s[sCount] <> '.') and (sCount > 0)do Dec(sCount);
  if (sCount < 1) then sCount := Length(s) + 1;
  s := Copy(s, 1, sCount - 1);
  // Isloate the numbers
  sCount := Length(s);
  while StrToIntDef(s[sCount], -1) <> -1 do Dec(sCount);
  s := Copy(s, sCount + 1, Length(s));
  // Remove 0 padding
  sCount := 1;
  if Length(s) > 0 then
  while s[sCount] = '0' do Inc(sCount);
  s := Copy(s, sCount, Length(s));
  Result := s;
end;



initialization
  {$I watcher_gapcheck_form.lrs}

end.

