{
  Copyright 2011-2012 Joe Justice (joe@joejustice.org)
 
  This file is part of LightNet: Cloud.

  LightNet: Cloud is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  LightNet: Cloud is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with LightNet: Cloud, see files COPYING.txt and COPYING.LESSER.txt 
  If not, see <http://www.gnu.org/licenses/>.

----------------------------------------------------------------------------

This is the form for downloading a scene's output from the FTP server

}

unit watcher_scenedownload_form;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  StdCtrls, ExtCtrls, watcher_filelister_form, lnc_utils;

type
  { TWatchSceneDownload }
  TWatchSceneDownload = class(TForm)
    Button_Go: TButton;
    ComboBox_Scene: TComboBox;
    Label1: TLabel;
    Memo_Listing: TMemo;
    OK: TButton;
    Cancel: TButton;
    SelectDirectoryDialog1: TSelectDirectoryDialog;
    procedure Button_GoClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure OKClick(Sender: TObject);
  private
    LocalRoot: string;
    { private declarations }
  public
    { public declarations }
  end; 

implementation
  uses lncwatcher_form;

{ TWatchSceneDownload }

procedure TWatchSceneDownload.FormCreate(Sender: TObject);

begin
end;

procedure TWatchSceneDownload.FormShow(Sender: TObject);
var
  WatchFileLister: TWatchFileLister;
  FileList, LineList: TStringList;
  i: integer;

begin
  if SelectDirectoryDialog1.Execute then
    begin
      LocalRoot := SelectDirectoryDialog1.FileName;
      WatchFileLister := TWatchFileLister.Create(self);
      WatchFileLister.GetListing(LnC_Settings.ftpSubDir + ftpPathDelim + '*.lws');
      if WatchFileLister.ShowModal = mrOK then
        begin
          FileList := TStringList.Create;
          FileList.Text := WatchFileLister.Listing;
          for i := 0 to FileList.Count - 1 do
            begin
              LineList := TStringList.Create;
              LineList.DelimitedText := FileList.ValueFromIndex[i];
              ComboBox_Scene.Items.Add(LnC_ftpExtractFileName(LineList.ValueFromIndex[8]));
              LineList.Free;
            end;
          FileList.Free;
        end;
      WatchFileLister.Free;
      if ComboBox_Scene.Items.Count = 0 then
        ComboBox_Scene.Items.Insert(0, 'No scenes found.')
      else
        Button_Go.Enabled := true;
      ComboBox_Scene.ItemIndex := 0;
    end
  else
    ModalResult := mrCancel;


end;

procedure TWatchSceneDownload.OKClick(Sender: TObject);
begin
  ModalResult := mrOK
end;

procedure TWatchSceneDownload.Button_GoClick(Sender: TObject);

var
  WatchFileLister: TWatchFileLister;
  i, i2 :integer;
  s, SceneName, SceneRGB, SceneAlpha, SceneRoot :string;
  TmpList, LineList: TStringList;

begin
  OK.Enabled := false;
  Cancel.Enabled := false;
  SceneName := ComboBox_Scene.Items.ValueFromIndex[ComboBox_Scene.ItemIndex];
  CreateDir (LocalRoot + PathDelim + LnC_GetOutputDir(SceneName, 'root'));
  CreateDir (LocalRoot + PathDelim + LnC_GetOutputDir(SceneName, 'rgb'));
  CreateDir (LocalRoot + PathDelim + LnC_GetOutputDir(SceneName, 'alpha'));
  SceneRGB := LnC_GetftpOutputDir(SceneName, 'rgb');
  SceneAlpha := LnC_GetftpOutputDir(SceneName, 'alpha');
  SceneRoot := LnC_GetftpOutputDir(SceneName, 'root');
  TmpList := TStringList.Create;
  Memo_Listing.Lines.Clear;
  // Get RGB output
  WatchFileLister := TWatchFileLister.Create(self);
  WatchFileLister.GetListing(LnC_Settings.ftpSubDir + ftpPathDelim + SceneRGB);
  if WatchFileLister.ShowModal = mrOK then
    begin
      TmpList.Text := WatchFileLister.Listing;
      // Remove empty lines
      for i := TmpList.Count - 1 downto 0 do
        begin
          if TmpList.ValueFromIndex[i] = '' then TmpList.Delete(i);
        end;
      // Replace full file listing with filename and path
      for i := 0 to TmpList.Count - 1 do
        begin
          LineList := TStringList.Create;
          LineList.DelimitedText := TmpList.ValueFromIndex[i];
          // In case the file has spaces in the name
          s := '';
          for i2 := 8 to LineList.Count - 1 do
            s := s + ' ' + LineList.ValueFromIndex[i2];
          // In case the FTP server returns a full path
          s := LnC_ftpExtractFileName(s);
          if (Copy(LineList.ValueFromIndex[0], 1, 1) <> 'd') and (s <> '') then
            s := LocalRoot + PathDelim + LnC_GetOutputDir(SceneName, 'rgb') + PathDelim + trim(s) + '|' + LnC_Settings.ftpSubDir + ftpPathDelim + SceneRGB + ftpPathDelim + trim(s)
          else
            s:='';
          TmpList.Delete(i);
          TmpList.Insert(i, s);
          LineList.Free;
        end;
    end;
  // Remove empty lines
  for i := TmpList.Count - 1 downto 0 do
    begin
      if TmpList.ValueFromIndex[i] = '' then TmpList.Delete(i);
    end;
  // Add files to memo list
  for i := 0 to TmpList.Count - 1 do
    begin
      Memo_Listing.Append(TmpList.ValueFromIndex[i]);
    end;
  WatchFileLister.Free;
  TmpList.Clear;

  // Get alpha output
  WatchFileLister := TWatchFileLister.Create(self);
  WatchFileLister.GetListing(LnC_Settings.ftpSubDir + ftpPathDelim + SceneAlpha);
  if WatchFileLister.ShowModal = mrOK then
    begin
      TmpList.Text := WatchFileLister.Listing;
      // Remove empty lines
      for i := TmpList.Count - 1 downto 0 do
        begin
          if TmpList.ValueFromIndex[i] = '' then TmpList.Delete(i);
        end;
      // Replace full file listing with filename and path
      for i := 0 to TmpList.Count - 1 do
        begin
          LineList := TStringList.Create;
          LineList.DelimitedText := TmpList.ValueFromIndex[i];
          // In case the file has spaces in the name
          s := '';
          for i2 := 8 to LineList.Count - 1 do
            s := s + ' ' + LineList.ValueFromIndex[i2];
          // In case the FTP server returns a full path
          s := LnC_ftpExtractFileName(s);
          if (Copy(LineList.ValueFromIndex[0], 1, 1) <> 'd') and (s <> '') then
            s := LocalRoot + PathDelim + LnC_GetOutputDir(SceneName, 'alpha') + PathDelim + trim(s) + '|' + LnC_Settings.ftpSubDir + ftpPathDelim + SceneAlpha + ftpPathDelim + trim(s)
          else
            s:='';
          TmpList.Delete(i);
          TmpList.Insert(i, s);
          LineList.Free;
        end;
    end;
  // Remove empty lines
  for i := TmpList.Count - 1 downto 0 do
    begin
      if TmpList.ValueFromIndex[i] = '' then TmpList.Delete(i);
    end;
  // Add files to memo list
  for i := 0 to TmpList.Count - 1 do
    begin
      Memo_Listing.Append(TmpList.ValueFromIndex[i]);
    end;
  WatchFileLister.Free;
  // Get scene and log files
  WatchFileLister := TWatchFileLister.Create(self);
  WatchFileLister.GetListing(LnC_Settings.ftpSubDir + ftpPathDelim + Copy(SceneName, 1, Length(SceneName) - 4) + '*');
  if WatchFileLister.ShowModal = mrOK then
    begin
      TmpList.Text := WatchFileLister.Listing;
      // Remove empty lines
      for i := TmpList.Count - 1 downto 0 do
        begin
          if TmpList.ValueFromIndex[i] = '' then TmpList.Delete(i);
        end;
      // Replace full file listing with filename and path
      for i := 0 to TmpList.Count - 1 do
        begin
          LineList := TStringList.Create;
          LineList.DelimitedText := TmpList.ValueFromIndex[i];
          // In case the file has spaces in the name
          s := '';
          for i2 := 8 to LineList.Count - 1 do
            s := s + ' ' + LineList.ValueFromIndex[i2];
          // In case the FTP server returns a full path
          s := LnC_ftpExtractFileName(s);
          if (Copy(LineList.ValueFromIndex[0], 1, 1) <> 'd') and (s <> '') then
            s := LocalRoot + PathDelim + trim(s) + '|' + LnC_Settings.ftpSubDir + ftpPathDelim + trim(s)
          else
            s:='';
          TmpList.Delete(i);
          TmpList.Insert(i, s);
          LineList.Free;
        end;
    end;
  // Remove empty lines
  for i := TmpList.Count - 1 downto 0 do
    begin
      if TmpList.ValueFromIndex[i] = '' then TmpList.Delete(i);
    end;
  // Add files to memo list
  for i := 0 to TmpList.Count - 1 do
    begin
      Memo_Listing.Append(TmpList.ValueFromIndex[i]);
    end;
  WatchFileLister.Free;

  TmpList.Free;
  OK.Enabled := true;
  Cancel.Enabled := true;
end;


initialization
  {$I watcher_scenedownload_form.lrs}

end.

